#
# spec file for package patterns-base
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%bcond_with betatest
Name:           patterns-base
Version:        20190206
Release:        4.1
Summary:        Patterns for Installation (base patterns)
License:        MIT
Group:          Metapackages
URL:            https://github.com/openSUSE/patterns
Source0:        %{name}-rpmlintrc
Source1:        pattern-definition-32bit.txt
Source2:        create_32bit-patterns_file.pl
BuildRequires:  patterns-rpm-macros

%description
This is an internal package that is used to create the patterns as part
of the installation source setup.  Installation of this package does
not make sense.

This particular package contains all the base / core patterns (and those that don't fit well anywhere else).

################################################################################

# bsc#1088669 - only provide 32bit pattern for 64bit intel
%ifarch x86_64
%package 32bit
%pattern_basetechnologies
Summary:        32-Bit Runtime Environment
Group:          Metapackages
Provides:       pattern() = 32bit
Provides:       pattern-icon() = pattern-cli
Provides:       pattern-order() = 1180
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-32bit = %{version}
Provides:       patterns-openSUSE-x86 = %{version}
Obsoletes:      patterns-openSUSE-32bit < %{version}
Obsoletes:      patterns-openSUSE-x86 < %{version}
%else
Provides:       patterns-sled-32bit = %{version}
Provides:       patterns-sles-32bit = %{version}
Obsoletes:      patterns-sled-32bit < %{version}
Obsoletes:      patterns-sles-32bit < %{version}
%endif

%description 32bit
This will install the 32-bit variant of all selected patterns. This allows to execute 32-bit software.

%files 32bit
%dir %{_docdir}/patterns
%{_docdir}/patterns/32bit.txt
%endif

################################################################################

%package apparmor
%pattern_basetechnologies
Summary:        AppArmor
Group:          Metapackages
Provides:       pattern() = apparmor
Provides:       pattern-icon() = pattern-apparmor
Provides:       pattern-order() = 1100
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-apparmor = %{version}
Obsoletes:      patterns-openSUSE-apparmor < %{version}
%else
Provides:       patterns-sled-apparmor = %{version}
Provides:       patterns-sles-apparmor = %{version}
Obsoletes:      patterns-sled-apparmor < %{version}
Obsoletes:      patterns-sles-apparmor < %{version}
%endif
Requires:       pattern() = minimal_base
%if 0%{?is_opensuse}
Recommends:     pattern() = apparmor_opt
%endif

Requires:       apparmor-abstractions
Requires:       apparmor-parser
Requires:       apparmor-profiles
Recommends:     apparmor-docs
Recommends:     apparmor-utils
Recommends:     yast2-apparmor
Suggests:       pam_apparmor
%if 0%{?is_opensuse}
Requires:       audit
%else
Recommends:     audit
%endif

%description apparmor
AppArmor is an application security framework that provides mandatory access control for programs. It protects from exploitation of software flaws and compromised systems. It offers an advanced tool set that automates the development of per-program application security without requiring additional knowledge.

%files apparmor
%dir %{_docdir}/patterns
%{_docdir}/patterns/apparmor.txt

################################################################################

%if 0%{?is_opensuse}
%package apparmor_opt
%pattern_basetechnologies
Summary:        AppArmor
Group:          Metapackages
Provides:       pattern() = apparmor_opt
Provides:       pattern-extends() = apparmor
Provides:       pattern-icon() = pattern-apparmor
Provides:       pattern-order() = 1080
# Keep these grouped
Provides:       patterns-openSUSE-apparmor_opt = %{version}
Obsoletes:      patterns-openSUSE-apparmor_opt < %{version}
Requires:       pattern() = minimal_base

Requires:       apparmor-docs

%description apparmor_opt
AppArmor is an application security framework that provides mandatory access control for programs. It protects from exploitation of software flaws and compromised systems. It offers an advanced tool set that automates the development of per-program application security without requiring additional knowledge.

%files apparmor_opt
%dir %{_docdir}/patterns
%{_docdir}/patterns/apparmor_opt.txt
%endif

################################################################################

%package basesystem
%pattern_basetechnologies
Summary:        Minimal Base System (alias pattern for base)
Group:          Metapackages
Provides:       pattern() = basesystem
Provides:       pattern-icon() = pattern-basis
Requires:       pattern() = minimal_base

%description basesystem
This is the base runtime system.  It contains only a minimal multiuser booting system. For running on real hardware, you need to add additional packages and pattern to make this pattern useful on its own.

%files basesystem
%dir %{_docdir}/patterns
%{_docdir}/patterns/basesystem.txt

################################################################################

%package base
%pattern_basetechnologies
Summary:        Minimal Base System
Group:          Metapackages
Provides:       pattern() = base
Provides:       pattern-icon() = pattern-basis
Provides:       pattern-order() = 1030
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-base = %{version}
Obsoletes:      patterns-openSUSE-base < %{version}
%else
Provides:       patterns-sles-base
Provides:       patterns-sles-minimal
Obsoletes:      patterns-sles-base < %{version}
Obsoletes:      patterns-sles-minimal < %{version}
%endif
Requires:       pattern() = minimal_base

Requires:       e2fsprogs
Requires:       kmod
Requires:       openssh
Requires:       polkit
Requires:       polkit-default-privs
Requires:       shadow
Requires:       util-linux
Requires:       which
%if ! 0%{?is_opensuse}
Requires:       systemd-coredump
%endif
# Add some static base tool in case system explodes; Recommend only, as users are free to uninstall it
Recommends:     busybox-static
Recommends:     bash-completion
Recommends:     btrfsprogs
Recommends:     ca-certificates-mozilla
Recommends:     chrony
Recommends:     cron
# we rely on cron for daily/hourly
Recommends:     cronie
Recommends:     grub2
Recommends:     glibc-locale
Recommends:     systemd-sysvinit
Recommends:     snapper

%if 0%{?is_opensuse}
# get it branded
Recommends:     branding-openSUSE
%else
Recommends:     branding-SLE
%endif
%ifarch x86_64
Recommends:     efibootmgr
Recommends:     shim
Recommends:     grub2-x86_64-efi
%endif
%ifarch aarch64
Recommends:     shim
Recommends:     grub2-arm64-efi
%endif
%ifarch %arm
Recommends:     efibootmgr
Recommends:     grub2-arm-efi
%endif
%ifarch ppc ppc64 ppc64le
%if !0%{?is_opensuse}
Recommends:     lshw
Recommends:     lsvpd
%endif
%endif
%ifarch ppc64 ppc64le
# bsc#1098849
Requires:       ppc64-diag
%endif

%description base
This is the base runtime system.  It contains only a minimal multiuser booting system. For running on real hardware, you need to add additional packages and pattern to make this pattern useful on its own.

%files base
%dir %{_docdir}/patterns
%{_docdir}/patterns/base.txt

################################################################################

%if 0%{?is_opensuse}
%package console
%pattern_basetechnologies
Summary:        Console Tools
Group:          Metapackages
Provides:       pattern() = console
Provides:       pattern-icon() = pattern-cli
Provides:       pattern-order() = 1120
Provides:       pattern-visible()
# Keep these grouped
Provides:       patterns-openSUSE-console = %{version}
Obsoletes:      patterns-openSUSE-console < %{version}
Requires:       pattern() = enhanced_base
Recommends:     pattern() = yast2_basis

Recommends:     at
Recommends:     bc
Recommends:     ed
Recommends:     emacs-nox
Recommends:     libyui-ncurses
Recommends:     libyui-ncurses-pkg
Recommends:     mc
Recommends:     mosh
Recommends:     mtools
Recommends:     sensors
Recommends:     susepaste
Recommends:     susepaste-screenshot
Recommends:     tmux
Recommends:     w3m
Suggests:       alpine
Suggests:       bsd-games
Suggests:       cnetworkmanager
Suggests:       convert
Suggests:       dar
Suggests:       ding
Suggests:       gcal
Suggests:       grepmail
Suggests:       irssi
Suggests:       lftp
Suggests:       links
Suggests:       lynx
Suggests:       minicom
Suggests:       mlocate
Suggests:       mutt
Suggests:       ncftp
Suggests:       pico
Suggests:       pinfo
Suggests:       slrn
Suggests:       units
Suggests:       vlock

%description console
Applications useful for those using the console and no graphical desktop environment.

%files console
%dir %{_docdir}/patterns
%{_docdir}/patterns/console.txt
%endif

################################################################################

%package documentation
%pattern_documentation
Summary:        Help and Support Documentation
Group:          Metapackages
Provides:       pattern() = documentation
Provides:       pattern-icon() = pattern-documentation
Provides:       pattern-order() = 1005
Provides:       pattern-visible()
Requires:       pattern() = basesystem
%if !0%{?is_opensuse}
Provides:       patterns-sled-documentation
Obsoletes:      patterns-sles-documentation < %{version}
%endif

Recommends:     info2html
Recommends:     man-pages
# note pam is in every install so no point in using packageand
Recommends:     pam-doc
Recommends:     susehelp
Recommends:     susehelp_en
%if !0%{?is_opensuse}
Recommends:     sled-admin_en-pdf
Recommends:     sled-gnomeuser_en-pdf
Recommends:     sled-manuals_en-pdf
Recommends:     sled-security_en-pdf
Recommends:     sled-tuning_en-pdf
%endif

%description documentation
Help and Support Documentation

%files documentation
%dir %{_docdir}/patterns
%{_docdir}/patterns/documentation.txt

################################################################################

%package enhanced_base
%pattern_basetechnologies
Summary:        Enhanced Base System
Group:          Metapackages
Provides:       pattern() = enhanced_base
Provides:       pattern-icon() = pattern-basis
Provides:       pattern-order() = 1060
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-enhanced_base = %{version}
Obsoletes:      patterns-openSUSE-enhanced_base < %{version}
%endif
Requires:       pattern() = base
Recommends:     pattern() = apparmor
Recommends:     pattern() = sw_management
Recommends:     pattern() = yast2_basis
%if 0%{?is_opensuse}
Recommends:     pattern() = enhanced_base_opt
%endif

Recommends:     aaa_base-extras
# getfacl and setfacl
Recommends:     acl
# #302569
Recommends:     alsa-plugins
# getattr and setattr
Recommends:     attr
Recommends:     autofs
Recommends:     bind-utils
Recommends:     binutils
# compressor is interesting
Recommends:     bzip2
# #375103
Recommends:     cifs-utils
Recommends:     command-not-found
Recommends:     cpio
Recommends:     cpupower
Recommends:     cronie
Recommends:     cryptsetup
# cups server for remote printing queues
Recommends:     cups
# printing considered cool
Recommends:     cups-client
Recommends:     curl
Recommends:     cyrus-sasl
Recommends:     cyrus-sasl-crammd5
Recommends:     cyrus-sasl-digestmd5
Recommends:     cyrus-sasl-gssapi
Recommends:     cyrus-sasl-plain
# bnc#430895
# cyrus-sasl-saslauthd
# delta rpms are considered cool for updates
Recommends:     deltarpm
Recommends:     diffutils
Recommends:     dos2unix
Recommends:     e2fsprogs
Recommends:     ethtool
Recommends:     file
Recommends:     fillup
Recommends:     findutils
# firewall by default
Recommends:     firewalld
Recommends:     fuse
Recommends:     gawk
Recommends:     gettext-runtime
Recommends:     glibc-locale
Recommends:     gpart
Recommends:     gpg2
Recommends:     gpm
Recommends:     grep
Recommends:     gzip
Recommends:     hdparm
Recommends:     hwinfo
Recommends:     info
Recommends:     initviocons
# /bin/ip considered useful
Recommends:     iproute2
# ping is required for network tests
Recommends:     iputils
Recommends:     irqbalance
Recommends:     joe
Recommends:     kmod-compat
# #303857
Recommends:     kpartx
Recommends:     krb5
# pager
Recommends:     less
Recommends:     logrotate
Recommends:     lsscsi
Recommends:     mailx
# man by default (#304687)
Recommends:     man
# needed for detecting software raid - required by yast2-storage too
Recommends:     mdadm
Recommends:     multipath-tools
# split out of ncurses
Recommends:     ncurses-utils
Recommends:     net-snmp
Recommends:     netcat-openbsd
Recommends:     netcfg
Recommends:     nfs-client
Recommends:     nfsidmap
Recommends:     nscd
# mount NTFS rw (bsc#1087242)
Recommends:     ntfs-3g
Recommends:     ntfsprogs
Recommends:     openslp
# we want a ssh server to be available
Recommends:     openssh
# TODO: should this be in more places
Recommends:     pam-config
Recommends:     parted
Recommends:     pciutils
Recommends:     pciutils-ids
Recommends:     perl-Bootloader
Recommends:     perl-base
Recommends:     pinentry
Recommends:     postfix
# fuser (psmisc) by default (#304694)
Recommends:     psmisc
Recommends:     rsync
# we still want /var/log/messages as all of the docu refers to it
# TODO: if we still want it everywhere it should move back to base
Recommends:     rsyslog
# Bug 424707 - Feature "Command not found" for openSUSE by default
Recommends:     scout
Recommends:     screen
Recommends:     sed
Recommends:     sg3_utils
Recommends:     smartmontools
Recommends:     sudo
Recommends:     sysconfig
#SUSE hardware tunings
Recommends:     system-tuning-common-SUSE
Recommends:     systemd-sysvinit
Recommends:     time
Recommends:     timezone
Recommends:     translation-update
Recommends:     udev
# autoconfig new printers - bnc#808014
Recommends:     udev-configure-printer
# lsusb is good for debugging USB devices - #401593
Recommends:     usbutils
# Our editor of choice
Recommends:     vim
Recommends:     wget
Recommends:     xz
Recommends:     zisofs-tools
# DELL computers mainly #403270, but #441079
Suggests:       biosdevname
Suggests:       cpupower
# #437252
Suggests:       pam_ssh
Suggests:       xfsprogs
Suggests:       zip
%ifarch aarch64 %ix86 x86_64
Recommends:     dmidecode
%endif
# Current systems suffer from entropy starvation
%ifarch aarch64 %ix86 x86_64 ppc64 ppc64le
Recommends:     haveged
%endif
%ifarch ppc
Recommends:     hfsutils
%endif
%ifarch ppc
# #303737
Recommends:     mouseemu
Recommends:     pdisk
Recommends:     powerpc32
%endif
# openSUSE Branding packages first
%if 0%{?is_opensuse}
# we want a branded grub2 too #757683
Recommends:     grub2-branding-openSUSE
Recommends:     plymouth
Recommends:     plymouth-branding-openSUSE
Recommends:     release-notes-openSUSE
%else
Recommends:     grub2-branding-SLE
Recommends:     plymouth
%endif
# Other packages we have in openSUSE and not SLE-15
%if 0%{?is_opensuse}
Recommends:     dmraid
Recommends:     dosfstools
Recommends:     ifplugd
Recommends:     klogd
# boo#1034493
Recommends:     nano
Recommends:     openldap2-client
Recommends:     prctl
# fuser (psmisc) by default (#304694)
Recommends:     psmisc
Recommends:     smp_utils
# useful for debugging
Recommends:     strace
Recommends:     syslinux
# having a ftp command line client is good for moving log files
Recommends:     tnftp
Recommends:     tuned
Recommends:     wireless-tools
Recommends:     wol
%ifarch %ix86 x86_64
Recommends:     acpica
%endif
%ifarch x86_64
Recommends:     mcelog
%endif
%ifarch aarch64 x86_64
Recommends:     numactl
%endif
%ifarch %ix86 x86_64
Recommends:     ucode-amd
Recommends:     ucode-intel
%endif
%endif

%description enhanced_base
This is the enhanced base runtime system with lots of convenience packages.

%files enhanced_base
%dir %{_docdir}/patterns
%{_docdir}/patterns/enhanced_base.txt

################################################################################

%if 0%{?is_opensuse}
%package enhanced_base_opt
%pattern_basetechnologies
Summary:        Enhanced Base System
Group:          Metapackages
Provides:       pattern() = enhanced_base_opt
Provides:       pattern-extends() = enhanced_base
Provides:       pattern-icon() = pattern-software-management
Provides:       pattern-order() = 1040
# Keep these grouped
Provides:       patterns-openSUSE-enhanced_base_opt = %{version}
Obsoletes:      patterns-openSUSE-enhanced_base_opt < %{version}

Recommends:     joe
Recommends:     mpt-status
Recommends:     perl-TermReadLine-Gnu
Recommends:     prctl
Recommends:     procinfo
Recommends:     procmail
Recommends:     providers
Recommends:     setserial
Recommends:     sharutils
Recommends:     spax
Recommends:     strace
Recommends:     tcpdump
Recommends:     telnet
Recommends:     terminfo
Recommends:     vlan
Recommends:     wol
Suggests:       acpid
Suggests:       cracklib-dict-full
# needed as soon as you want to do kerberos authentication
Suggests:       cyrus-sasl-gssapi
Suggests:       delayacct-utils
Suggests:       groff
Suggests:       hfsutils
# bnc#388570
Suggests:       kerneloops
Suggests:       man-pages
Suggests:       man-pages-posix
Suggests:       ocfs2-tools
Suggests:       pwgen
Suggests:       unzip
Suggests:       w3m-el
# delta apply
Suggests:       xdelta
# tool for xfs
Suggests:       xfsdump
# #754959
%ifarch %ix86 x86_64
Suggests:       hyper-v
%endif

%description enhanced_base_opt
This is the enhanced base runtime system with lots of convenience packages.

%files enhanced_base_opt
%dir %{_docdir}/patterns
%{_docdir}/patterns/enhanced_base_opt.txt
%endif

################################################################################

%package minimal_base
%pattern_basetechnologies
Summary:        Minimal Appliance Base
Group:          Metapackages
Provides:       pattern() = minimal_base
Provides:       pattern-icon() = pattern-basis
Provides:       pattern-order() = 5190
Provides:       pattern-visible()
# Keep these grouped
Provides:       patterns-openSUSE-minimal_base = %{version}
Obsoletes:      patterns-openSUSE-minimal_base < %{version}

Requires:       aaa_base
Requires:       bash
Requires:       coreutils
Requires:       device-mapper
Requires:       distribution-release
Requires:       dracut
Requires:       filesystem
Requires:       glibc
Requires:       iproute2
Requires:       kbd
Requires:       pam
Requires:       procps
Requires:       rpm
Requires:       sysconfig
Requires:       system-group-hardware
Requires:       system-group-wheel
Requires:       system-user-bin
Requires:       system-user-daemon
Requires:       system-user-nobody
Requires:       systemd
Requires:       udev
Requires:       zypper
# Note it makes no sense to recommend packages in minimal_base as it can't
# be installed with --no-recommends if your package can be Recommended rather
# then required it likely belongs in base and not here.
# rollback-helper is useful on Leap / SLE but not tumbleweed
%if 0%{?sle_version}
Requires:       rollback-helper
%endif
%if 0%{?is_opensuse}
Requires:       openSUSE-build-key
# There are two release packages in the repo, pick this over openSUSE-Tumbleweed-Kubic-release
Suggests:       openSUSE-release
%else
Requires:       SUSEConnect
Requires:       rollback_helper
Requires:       suse-build-key
%endif

%if 0%{?is_opensuse}
%description minimal_base
This is the minimal openSUSE runtime system. It is really a minimal system, you can login and a shell will be started, that's all. It is intended as base for Appliances.
%else
%description minimal_base
This is the minimal SLE runtime system. It is really a minimal system, you can login and a shell will be started, that's all. It is intended as base for Appliances.
%endif

%files minimal_base
%dir %{_docdir}/patterns
%{_docdir}/patterns/minimal_base.txt

################################################################################

%package transactional_base
%pattern_basetechnologies
Summary:        Transactional Base System
Group:          Metapackages
Provides:       pattern() = transactional_base
Provides:       pattern-icon() = pattern-kubic
Provides:       pattern-order() = 1050
Obsoletes:      pattern() = readonly_root_tools
Requires:       pattern() = base

Requires:       read-only-root-fs
Requires:       rebootmgr
Requires:       systemd-presets-branding-transactional-server
Requires:       transactional-update
Requires:       transactional-update-zypp-config
Recommends:     pattern() = enhanced_base
Suggests:       health-checker

%description transactional_base
This is the base system for a host updated by Transactional Updates. Includes Tools for systems with a read-only root filesystem.

%files transactional_base
%dir %{_docdir}/patterns
%{_docdir}/patterns/transactional_base.txt

################################################################################

%package sw_management
%pattern_basetechnologies
Summary:        Software Management
Group:          Metapackages
Provides:       pattern() = sw_management
Provides:       pattern-icon() = pattern-software-management
Provides:       pattern-order() = 1360
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-sw_management = %{version}
Obsoletes:      patterns-openSUSE-sw_management < %{version}
%endif
%if 0%{?sle_version}
Recommends:     pattern() = sw_management_x11

# Zypper is the basic sw_management stack for *SUSE
Requires:       zypper
Recommends:     lifecycle-data
Recommends:     zypper-lifecycle-plugin
%endif

%description sw_management
This pattern provides a graphical application and a command line tool for keeping your system up to date.

%files sw_management
%dir %{_docdir}/patterns
%{_docdir}/patterns/sw_management.txt

################################################################################

%if 0%{?is_opensuse}
%package update_test
%pattern_basetechnologies
Summary:        Tests for the Update Stack
Group:          Metapackages
Provides:       pattern() = update_test
Provides:       pattern-icon() = pattern-tests
Provides:       pattern-order() = 1380
Provides:       pattern-visible()
# Keep these grouped
Provides:       patterns-openSUSE-update_test = %{version}
Obsoletes:      patterns-openSUSE-update_test < %{version}

Recommends:     update-test-affects-package-manager
Recommends:     update-test-interactive
Recommends:     update-test-optional
Recommends:     update-test-reboot-needed
Recommends:     update-test-security
Recommends:     update-test-trival
%if %{with betatest}
Recommends:     aaa_base-malloccheck
Supplements:    %{name}-base
%endif

%description update_test
Packages used for testing that the update stack works.  These tiny packages do not have any functionality themselves.

%files update_test
%dir %{_docdir}/patterns
%{_docdir}/patterns/update_test.txt
%endif

################################################################################

%package x11
%pattern_graphicalenvironments
Summary:        X Window System
Group:          Metapackages
Provides:       pattern() = x11
Provides:       pattern-icon() = pattern-x11
Provides:       pattern-order() = 1800
Provides:       pattern-visible()
%if 0%{?is_opensuse}
Provides:       patterns-openSUSE-x11 = %{version}
Obsoletes:      patterns-openSUSE-x11 < %{version}
%endif
Requires:       pattern() = base
%if 0%{?is_opensuse}
Recommends:     pattern() = x11_enhanced
Recommends:     pattern() = x11_opt
%endif

Requires:       xorg-x11-fonts-core
Requires:       xorg-x11-server
Requires:       yast2-qt
Recommends:     dejavu-fonts
Recommends:     google-roboto-fonts
# choose icewm-default if you have a choice
# icewm-lite is too lightweight in new release
Recommends:     icewm-default
Recommends:     icewm-theme-branding
Recommends:     libyui-qt
Recommends:     libyui-qt-pkg
# Recommend lightdm so it gets installed by default
# rather then xdm bsc#1081760
Recommends:     lightdm
Recommends:     noto-sans-fonts
Recommends:     openssh-askpass-gnome
Recommends:     tigervnc
Recommends:     x11-tools
Recommends:     xdmbgrd
Recommends:     xorg-x11-Xvnc
Recommends:     xorg-x11-driver-video
Recommends:     xorg-x11-essentials
Recommends:     xorg-x11-fonts
Recommends:     xorg-x11-server-extra
Recommends:     xterm
Recommends:     xtermset
Recommends:     yast2-control-center

# bsc#1071953
%ifnarch s390 s390x
Requires:       xf86-input-libinput
Recommends:     xf86-input-vmmouse
Recommends:     xf86-input-wacom
%endif

%description x11
The X Window System provides the only standard platform-independent networked graphical window system bridging the heterogeneous platforms in today's enterprise: from network servers to desktops, thin clients, laptops, and handhelds, independent of operating system and hardware.

%files x11
%dir %{_docdir}/patterns
%{_docdir}/patterns/x11.txt

################################################################################

%package x11_enhanced
%pattern_graphicalenvironments
Summary:        X Window System
Group:          Metapackages
Provides:       pattern() = x11_enhanced
Provides:       pattern-icon() = pattern-x11
Provides:       pattern-order() = 1801
%if 0%{?is_opensuse}
Obsoletes:      patterns-openSUSE-x11 < %{version}
%else
Provides:       patterns-sled-minimal
Obsoletes:      patterns-sled-minimal < %{version}
%endif
Requires:       pattern() = enhanced_base
Requires:       pattern() = fonts
Requires:       pattern() = x11
Recommends:     pattern() = x11_yast
%if 0%{?is_opensuse}
Recommends:     pattern() = x11_opt
%endif

# 1057377
Requires:       glibc-locale
Requires:       xkeyboard-config
Requires:       xorg-x11-essentials
# make it possible to install firefox or chromium
Recommends:     web_browser
Recommends:     cabextract
Recommends:     command-not-found
Recommends:     dbus-1-glib
Recommends:     dbus-1-x11
Recommends:     dialog
Recommends:     fontconfig
Recommends:     fonts-config
Recommends:     fribidi
Recommends:     ghostscript-x11
# choose icewm-default if you have a choice
# icewm-lite is too lightweight in new release
Recommends:     icewm-default
# Recommend lightdm so it gets installed by default
# rather then xdm bsc#1081760
Recommends:     lightdm
Recommends:     numlockx
Recommends:     tigervnc
# #353229 - drag in empty replacements
Recommends:     translation-update
Recommends:     xauth
Recommends:     xdmbgrd
Recommends:     xkeyboard-config
Recommends:     xorg-x11-fonts
Recommends:     xorg-x11-fonts-core
Recommends:     yast2-control-center-gnome
Recommends:     yast2-scanner
# This will install Firefox if no other browser is selected
Suggests:       MozillaFirefox
Suggests:       MozillaFirefox-translations

%if 0%{?is_opensuse}
# #394406
Suggests:       desktop-data-openSUSE-extra
%else
Recommends:     MozillaFirefox-branding-SLE
Recommends:     desktop-data-SLE
%endif
%if 0%{?is_opensuse}
# people love having numlock configurable
Recommends:     numlockx
Recommends:     openssh-askpass
Recommends:     susepaste
Recommends:     susepaste-screenshot
Suggests:       gvim
Suggests:       hexchat
Suggests:       wpa_supplicant-gui
%endif

%description x11_enhanced
The X Window System provides the only standard platform-independent networked graphical window system bridging the heterogeneous platforms in today's enterprise: from network servers to desktops, thin clients, laptops, and handhelds, independent of operating system and hardware.

%files x11_enhanced
%dir %{_docdir}/patterns
%{_docdir}/patterns/x11_enhanced.txt

################################################################################

%if 0%{?is_opensuse}
%package x11_opt
%pattern_graphicalenvironments
Summary:        X Window System
Group:          Metapackages
Provides:       pattern() = x11_opt
Provides:       pattern-extends() = x11
Provides:       pattern-icon() = pattern-x11
Provides:       pattern-order() = 1680
# Keep these grouped
Provides:       patterns-openSUSE-x11_opt = %{version}
Obsoletes:      patterns-openSUSE-x11_opt < %{version}
Requires:       pattern() = enhanced_base
Requires:       pattern() = fonts
# needed e.g. for nvidia drivers
# #302566
Recommends:     x11-tools
Recommends:     xorg-x11-libX11-ccache
Suggests:       MozillaThunderbird
Suggests:       WindowMaker
Suggests:       WindowMaker-applets
Suggests:       WindowMaker-themes
Suggests:       unclutter
Suggests:       xlockmore
Suggests:       xorg-x11-driver-video-radeonhd
Suggests:       xorg-x11-driver-video-unichrome
# #389816
Suggests:       xorg-x11-server-sdk

%description x11_opt
The X Window System provides the only standard platform-independent networked graphical window system bridging the heterogeneous platforms in today's enterprise: from network servers to desktops, thin clients, laptops, and handhelds, independent of operating system and hardware.

%files x11_opt
%dir %{_docdir}/patterns
%{_docdir}/patterns/x11_opt.txt
%endif

################################################################################

%ifarch armv6hl armv7hl aarch64
%package x11_raspberrypi
%pattern_graphicalenvironments
Summary:        X Window System
Group:          Metapackages
Provides:       pattern() = x11_raspberrypi
Provides:       pattern-icon() = pattern-x11
Provides:       pattern-order() = 1803
Provides:       pattern-visible()

# Based on SUSE:SLE-15:GA:RaspberryPI/kiwi-templates-SLES15-JeOS/JeOS.kiwi
# Drivers
Recommends:     xf86-input-evdev
Recommends:     xf86-input-void
Recommends:     xf86-input-libinput
Recommends:     xf86-video-fbdev
# Other X11 packages
Suggests:       gconf2
Suggests:       xfd
Suggests:       xfontsel
Suggests:       xgamma
Suggests:       xhost
Suggests:       xinit
Suggests:       xinput
Suggests:       xkbcomp
Suggests:       xkbevd
Suggests:       xkbprint
Suggests:       xkbutils
Suggests:       xkeyboard-config
Suggests:       xkill
Suggests:       xlogo
Suggests:       xlsatoms
Suggests:       xlsclients
Suggests:       xlsfonts
Suggests:       xmag
Suggests:       xmessage
Suggests:       xmodmap
Suggests:       xorg-x11
Suggests:       xorg-x11-fonts-core
Suggests:       xorg-x11-server
Suggests:       xorg-x11-server-extra
Suggests:       xprop
Suggests:       xrandr
Suggests:       xrdb
Suggests:       xrestop
Suggests:       xscope
Suggests:       xscreensaver
Suggests:       xscreensaver-data
Suggests:       xset
Suggests:       xsetmode
Suggests:       xsetpointer
Suggests:       xsetroot
Suggests:       xterm
Suggests:       xtermset
Suggests:       xvinfo
Suggests:       xwd
Suggests:       xwininfo
Suggests:       xdm
Suggests:       x11-tools
Suggests:       x11perf
Suggests:       xauth
Suggests:       xbacklight
Suggests:       xclock
Suggests:       xconsole
Suggests:       xcursor-themes
Suggests:       xcursorgen
Suggests:       xdg-user-dirs
Suggests:       xdg-user-dirs-gtk
Suggests:       xdg-user-dirs-gtk-lang
Suggests:       xdg-utils
Suggests:       xdmbgrd
Suggests:       xdpyinfo
Suggests:       xev
Suggests:       xeyes
Suggests:       yast2-x11
# bsc#1095870
Recommends:     libyui-qt-pkg
Recommends:     yast2-control-center-qt
Suggests:       gtk3-metatheme-adwaita
Suggests:       gtk2-metatheme-adwaita
# Branding
%if ! 0%{?is_opensuse}
Suggests:       MozillaFirefox-branding-SLE
%endif
# X11/IceWM-specific packages
Recommends:     icewm
Suggests:       icewm-lite
Recommends:     icewm-default
Recommends:     icewm-theme-branding
Suggests:       polkit-gnome-lang
Recommends:     polkit-default-privs
# for IceWM taskbar mailbox icon (bsc#1093913)
Recommends:     mutt
Suggests:       mutt-lang

%description x11_raspberrypi
The X Window System provides the only standard platform-independent networked graphical window system bridging the heterogeneous platforms in today's enterprise: from network servers to desktops, thin clients, laptops, and handhelds, independent of operating system and hardware.

%files x11_raspberrypi
%dir %{_docdir}/patterns
%{_docdir}/patterns/x11_raspberrypi.txt
%endif

################################################################################

%prep

%build

%install
mkdir -p %{buildroot}%{_docdir}/patterns
%if 0%{?is_opensuse}
for i in apparmor base enhanced_base minimal_base \
     sw_management x11 x11_enhanced; do
%else
for i in apparmor base enhanced_base  minimal_base sw_management x11 x11_enhanced; do
%endif
    echo "This file marks the pattern $i to be installed." \
    >"%{buildroot}%{_docdir}/patterns/$i.txt"
    echo "This file marks the pattern $i to be installed." \
    >"%{buildroot}%{_docdir}/patterns/$i-32bit.txt"
done

# These packages don't generate a 32bit pattern
for i in \
%if 0%{?is_opensuse}
apparmor_opt basesystem console documentation enhanced_base_opt transactional_base update_test x11_opt \
%else
%ifnarch s390 s390x
32bit \
%endif
basesystem documentation \
%endif
%ifarch armv6hl armv7hl aarch64
x11_raspberrypi \
%endif
; do
    echo "This file marks the pattern $i to be installed." \
    >"%{buildroot}%{_docdir}/patterns/$i.txt"
done

%ifarch x86_64
echo "This file marks the pattern 32bit to be installed." \
>"%{buildroot}%{_docdir}/patterns/32bit.txt"
%endif

#
# This file is created at check-in time. Sorry for the inconsistent workflow :(
#
%include %{SOURCE1}

%changelog
* Fri Mar 15 2019 Fabian Vogt <fvogt@suse.com>
- Suggest openSUSE-release to guide zypper away from picking
  openSUSE-Tumbleweed-Kubic-release automatically
* Wed Mar  6 2019 Andreas Färber <afaerber@suse.de>
- Re-run pre_checkin.sh, updating -32bit patterns
- Add pattern x11_raspberrypi (fate#326509, jsc#SLE-4142)
  This is to cover the delta between SLES 15 GA and SP1 JeOS images
  to allow the user to recreate its IceWM desktop without requiring
  a lengthy list of packages.
  Extended to openSUSE on request by fcrozat.
- Insert comments to keep format_spec_file from reordering too much
* Mon Feb 25 2019 Dominique Leuenberger <dimstar@opensuse.org>
- Strip down minimal_base pattern:
  * Drop elfutils Requires: all tools from elfutils exist in
    binutils, without the eu- prefix.
  * Drop openssh requires: forcing a user to have openssh present no
    matter what, without a possibility to uninstall it, is not
    wanted.
  * Drop sysvinit requires: this is a dependency to systemd after
    all.
- Move systemd-coredump and busybox-static from requires in
  minimal_base to recommends in base (users are free to uninstall
  those tools and their use in e.g. containers is questionable).
* Tue Feb 19 2019 Frederic Crozat <fcrozat@suse.com>
- ppc64-diag is required on ppc64le (bsc#1098849).
* Thu Feb 14 2019 Simon Lees <sflees@suse.de>
- Pull in below changes made to SLE / openSUSE 15.1 patterns by
  kukuk@suse.de
- Rename base pattern to "Minimal Base System" to match description
  and make clear, that this is not a full base system.
- Rename minimal_base to "Minimal Appliance Base" to match description
  and avoid to have to patterns called "Base System" with different
  content.
* Tue Feb  5 2019 Simon Lees <sflees@suse.de>
- Version: 20190206
- Remove minimal_base_conflicts, its not in SLE or Leap anymore
  (boo#1103325)
- Change all the Recommends in minimal_base to Requires,
  minimal_base is always installed and can't be installed with
  - -no-recommends (boo#1103326)
- Hardware specific packages should be in base pattern not
  minimal_base (boo#1106405)
- busybox should be in minimal_base to allow recovery from a big
  system failure
- Remove some duplicate entries between base and minimal_base
- Suggest rather then recommend man-pages (boo#1116987)
- Require xorg-x11-essentials rather then recommending xorg-x11
  (boo#1121730)
- Recommend Web-web_browser and suggest Firefox to make it possible
  to install chromium instead
- Recommend rather then require multipath-tools, it is only in
  enhanced_base so requiring it doesn't make sense, if it Should
  be required it likely should be in base
- general cleanup, fix meta info spec-cleaner re arranged remove
  commented out packages
* Thu Jan 10 2019 Dirk Mueller <dmueller@suse.com>
- update version number to current date
* Mon Jan  7 2019 Dirk Mueller <dmueller@suse.com>
- recommend grub2-*arm*efi variants for arm/arm64 (bsc#1120804)
- recommend shim for aarch64
* Fri Jan  4 2019 Dominique Leuenberger <dleuenberger@suse.com>
- Recommend grub2-x86_64-efi by base pattern on x86_64: since we use
  the shim RPMs from Leap, there is currently no dependency on
  grub2-x86_64-efi defined. Once shim changes that, we can drop the
  recommends here.
* Tue Dec 11 2018 Dominique Leuenberger <dimstar@opensuse.org>
- Drop recommends to net-tools (no more tools of general
  usefulnes), popt and pcre (both are just libraries nowadays,
  which are pulled in by normal dependencies)
* Fri Dec  7 2018 Stasiek Michalski <hellcp@mailbox.org>
- Adjust icons to state of yast2-theme package
* Thu Nov 22 2018 Richard Brown <rbrown@suse.com>
- Add transactional-update-zypp-config alongside
  transactional-update (boo#1111319)
* Tue Oct 30 2018 Thomas.Blume@suse.com
- Recommend new package system-tuning-common-SUSE in
  enhanced_base
* Wed Oct 10 2018 Richard Brown <rbrown@suse.com>
- transactional_base now requires base, recommends enhanced_base
  (boo#1111426)
* Wed Oct  3 2018 sflees@suse.de
- nfs-doc is now installed with supplements
- remove the list of openSUSE only packages that was rejected for
  inclusion in SLE.
- Add bug numbers for TODO's
* Wed Oct  3 2018 sflees@suse.de
- Merge back in Leap 15 / SLE changes, important items from that
  packages changlog can be found below
  lnussel@suse.de
- explicitly include rsyslog in the base pattern to make sure
  logging goes to /var/log/messages as documented (bsc#1094841).
- require distribution-release instead of openSUSE-release to avoid conflicts
  when migrating to SLE (boo#1089031)
- do not recommend ntp also in enhanced_base (boo#162331)
- make everything in enhanced_base_opt really optional. It just
  increases the server install and SLE doesn't have the pattern at
  all either (bsc#1090189)
- move ordering of patterns-base-base after gnome_x11 (bsc#1091102)
- Don't generate 32bit patterns for readonly_root_tools
- Add create_32bit-patterns_file.pl as source
- guard some sle specifics with %%is_opensuse
  sflees@suse.de
- Rework x11 pattern bsc#1086663
- Recommend terminfo in minimal_base bsc#1081747
- rpm doesn't need to be in base and minimal_base_conflicts
- dont create transactional_base-32bit pattern.
- Remove grub variants from patterns bsc#1064265
- Don't install rollback helper on openSUSE (See why is this even
  in here.... thread on openSUSE project mailing list) boo#1086584
- add lightdm as a recommends to x11 pattern boo#1081760
- fix pattern names packgeand() lines of 32bit patterns (bsc#1079353)
- put chrony in base system (bsc#1072351)
- fix pattern() provides to use %%2d instead of dash to avoid rpm
  parsing the string as name-version (bsc#1079353)
- don't recommend insserv-compat as anything sysvinit is obsolete
- Add lshw
- lsvpd is only relevant on ppc systems
- Provide chrony in base patterns (bsc#1072351, fate#323432)
- Install pp64-diag and lsvpd by default (bsc#1072351)
- Add obsoletes across all patterns (bsc#1071761)
- minimal_base doesn't need to be visable in yast
- Basesystem and Documentation don't need 32bit patterns
- base pattern now obsoletes and provides Minimal to make
  SLES-12 upgrades smoother (bsc#1062164)
- x11_enhanced pattern now obsoletes and provides Minimal to make
  SLED-12 upgrades smoother (bsc#1062164)
- Source pattern-definition-32bit.txt correctly
- Bring back in other patterns, from sle
  * console
  * enhanced_base
  * enhanced_base_opt
  * sw_management
- Update enhanced_base and x11_enhanced based off SLES minimal and x11_enhanced
  patterns
- Split the following patterns into patterns-enhanced-base
  * console
  * enhanced_base
  * enhanced_base_opt
  * sw_management
- Remove minimal_base_conflicts (not in SLE we can probably live without)
- Major changes to X11 package, to line up with SLES, new package will be
  added in patterns-enhanced-base to keep openSUSE/SLED compaitible
- Add documentation from SLE
- Copy 32bit support from SLE (x86 package removed)
- slight change to apparmor packages to match sle
- recommend salt-minion only on SLE
- Remove ppc only requires from base
  fcrozat@suse.com
- Do not recommends on SLE patterns which aren't shipped on SLE.
- Ensure recommends on ntfs-3g and ntfsprogs are enabled on SLE
  (bsc#1087242).
- Ensure xorg-x11-driver-input is required for x11 pattern, except
  on s390/390x where it doesn't exist.
- Recommends plymouth on SLE (bsc#1067481).
- Ensure openSUSE patterns aren't provided / obsoletes when
  building for SLE.
- Add Recommend to enhanced_base: OpenIPMI bash-completion cpp
  cryptconfig expect ipmitool lvm2 m4 make mksh mutt quota
  supportutils sysfsutils tcsh w3m lsof psmisc sudo.
- Add grub2-branding-SLE recommends on enhanced_base on SLE.
- Fix description of minimal_base on SLE.
  behlert@suse.de
- drop patterns-base-32bit pattern for s390 and s390x [bsc#1088669]
- added timezone as recommended to minimal_base pattern
  [bsc#1085075]
- ethtool is available on SLE 15, moving out of "opensuse only"
  section [bsc#1087354]
  okurz@suse.com
- put cron in base system (bsc#1072602)
  fvogt@suse.com
- Don't require apparmor-docs, only recommend it
- Require xorg-x11-fonts-core instead of xorg-x11-fonts
- Don't require xorg-x11, only recommend it
  kukuk@suse.de
- Remove salt-minion from base [bsc#1064266]
- Move tar to minimal pattern, too many low level tools assume
  tar is installed by default
- Rename xterm-bin back to xterm, we still have xterm.rpm
- Remove libnss_compat2 (dropped, still part of glibc) and
  libnss_nis2 (needs to be together with ypbind)
- Remove again added pam-modules, this package deprecated since
  many years.
  coolo@suse.com
- Recommend glibc-locale from minimal_base not enhanced_base,
  we install in en_US, so we need a locale one way or the other.
  Requiring it from enhanced_base now to make sure it's part of
  regular installations (but can be disabled for e.g. chroots
  with C locale - bsc#1057377)
- Do not provide x11 in x11_enhanced but require it
- Move basic X11 apps to x11 pattern and Firefox to x11_enhanced
- Remove tcl and tk from default installation
  fbui@suse.com
- Don't install systemd-coredump by default on Leap (bsc#1083849)
- Add systemd-coredump to the list of recommended packages of miminal_base
  Latest systemd package splitted off its coredump management facility
  into a sub-package. Recommend this package so this functionnality is
  still available by default on SLE (but will be disabled on Leap, see
  bsc#1083849).
  Also give the possibility to block it by using a soft dep
  (Recommends:). This might be needed on live images for example where
  space is rather low.
* Thu Sep 27 2018 antoine.belvire@opensuse.org
- No longer recommend missing packages (boo#1104264):
  * genisoimage
  * ksymoops
  * system-group-trusted
* Tue Sep 25 2018 kukuk@suse.de
- Remove libnss_usrfiles from transactional_base, not needed
  anymore.
* Fri Aug 31 2018 lnussel@suse.de
- add lightdm as a recommends to x11 pattern boo#1081760
* Tue Aug 21 2018 tchvatal@suse.com
- Drop recode from recommends as it was droped from distro
  bsc#1104264
- Drop cryptconfig from suggest as it was removed from distro
* Thu Jul 26 2018 rbrown@suse.com
- Remove btrfsmaintenance from patterns-base (boo#1063638)
* Wed Jul 18 2018 rbrown@suse.com
- Make transactional_base pattern available for SLE (fate#326327)
* Tue Jun  5 2018 dimstar@opensuse.org
- Only recommend rollback-helper if sle_version is defined: The
  helper makes sense on Leap and SLE, where one can register with
  SCC, but not for Tumbleweed.
* Wed Apr 25 2018 tchvatal@suse.com
- Add busybox to base patterns in case your system explodes, in
  past we had sash doing the same
* Wed Apr 11 2018 rbrown@suse.com
- Rename readonly_root_tools to transactional_base (boo#1089095)
* Tue Apr 10 2018 rbrown@suse.com
- Correct transactional server role pattern to use enhanced_base
  (boo#1088884)
* Tue Apr  3 2018 dimstar@opensuse.org
- Recommend bash-completion by the base system (not minimal_base):
  most users will expect it to be present and until recently, it
  was in a recommended chain of systemd (boo#1087710).
* Wed Mar 28 2018 fbui@suse.com
- Add systemd-coredump to the list of recommended of miminal_base
  Latest systemd package split off its coredump management facility
  into a sub-package. Recommend this package so this functionnality is
  still available by default on Factory but give the possibility to
  block it by using a soft dep (Recommends:). This might be needed on
  live images for example where space is rather low.
* Tue Mar  6 2018 rbrown@suse.com
- Introduce readonly_root_tools pattern for Read-Only Root
  Filesystem (boo#1084149)
* Thu Jan 11 2018 rbrown@suse.com
- Remove finger from enahnced_base_opt as functionality already
  provided by pinky in coreutils.
* Thu Dec 14 2017 dimstar@opensuse.org
- Replace SuSEfirewall2 with firewalld in enhanced_base
  (fate#323460).
* Mon Dec 11 2017 dimstar@opensuse.org
- Switch ntp-client from ntp to chrony: replace corresponding
  recommends in console and enhanced_base pattern (FATE#323432).
* Thu Nov  9 2017 lnussel@suse.de
- move plymouth from base to X11. No need to have a fancy boot splash that
  pulls in extra deps for a text console (boo#1066510)
* Mon Sep 25 2017 sflees@suse.de
- Swap back to SuSEfirewall2, its not fully intergrated everywhere
* Tue Sep 19 2017 sflees@suse.de
- set many *_opt patterns to only build on openSUSE
- Add some is_opensuse sections for packages not in SLE
- Move certain packages from * to *_opt when they are not in SLE
  * x11
  * xorg-x11-libX11-ccache
  * xorg-x11-driver-video-radeonhd
  * xorg-x11-driver-video-unichrome
  * xterm (xterm-bin added in place)
  * x11-tools
- Move packages from x11-opt to x11 so x11-opt is openSUSE only
  * tk
  * xtermset
- Rework of enhanced_base
  * Move from enhanced_base to enhanced_base_opt
  * Move from enhanced_base_opt to enhanced_base
  * cpupower
  * smartmontools
  * screen
  * xz (now recommended rather then suggested)
  * rsync
  * haveged
  * mcelog
  * zip
  * initviocons
  * zisofs-tools
  * irqbalance
  * pam_ssh
  * xfsprogs
  * dos2unix
  * Move or remove the following packages to console
  * lynx
  * at
  * bc
  * ed
  * w3m
- Remove certain packages from patterns where they are not shipped
  in SLE15
  * minimal_base
  * insserv
  * base
  * systemd-logger
  * console
  * makedev
  * par
  * enhanced_base / enhanced_base_opt
  * db-utils
  * jfsutils
  * libatm1
  * lsb-release
  * ppp
  * pptp
  * utempter (only lib in factory)
  * reiserfs
  * ypbind
  * yptools
  * x11
  * freeglut (dropped from factory)
  * fvwm2
  * fvwm-themes
  * gv
  * mmv
  * pmidi
  * suspend
  * xorg-x11-driver-input (compatability meta package)
  * xine-ui
  * xosd
  * xosview
  * xpdf-poppler (dropped from factory)
  * wine
  * x11_opt
  * freeglut (dropped from factory)
- Remove the 32 and 64bit patterns
- move ntp work around for boo#936378 to patterns-yast
- remove systemd from enhanced_base_opt (already included elsewhere)
- swap SuSEfirewall2 to firewalld following SLE-15
- no longer recommend alternate shells (users who want to change know how to
  install them)
  * ksh
  * tcsh
  * zsh
- Recommend nano in enhanced_base boo#1034493
- Console pattern only recommends things not requires them
- tgt removed (in yast where it shoud be)
- open-iscsi just needs to be in the media patterns boo#393589
* Mon Sep 18 2017 dleuenberger@suse.com
- Add libnss_compat2 and libnss_nis2 to minimal_base: they were
  integrated into glibc before.
* Wed Aug 30 2017 dimstar@opensuse.org
- Introduce a wrapper pattern 'basesystem'. One pattern cannot
  provide two names. This is likely to disappear soon again, but
  helps for an easier migration/cleanup run (and fixes
  boo#1049997).
* Thu Jul 13 2017 dimstar@opensuse.org
- Move glibc-32bit and glibc-locale-32bit recommends from
  base_enhanced to x86 (boo#947161).
- No longer recommend nss-mdns-32bit by the x11 pattern.
- Mark 64bit pattern as invisible.
* Thu Jul  6 2017 dimstar@opensuse.org
- Make pattern minimal_base visible: there are quire some scripts
  relying on this to install containers.
* Mon Jul  3 2017 dimstar@opensuse.org
- Drop sash recommends: sash is being removed from Tumbleweed.
* Fri Jun 16 2017 tchvatal@suse.com
- Drop xinetd from patterns as if someone really needs it they should
  rather pick it up. Today soultion for the problem is the systemd
  socket service.
* Thu Jun  8 2017 dimstar@opensuse.org
- Rename pattern minimal_base-conflicts to minimal_base_conflicts:
  we fall into a neat trap with
    Provides: pattern() = minimal_base-conflicts
  As this is translated to RPM lingo, meaning 'minimal_base' is the
  'version' and 'conflicts' the 'release'; so anything requiring
  pattern() = minimal_base is satisfied by minimal_base-conflicts
  directly too (boo#1043348).
* Thu May 18 2017 lnussel@suse.de
- Recommend zypper-lifecycle-plugin in sw_management for stable
  distributions
* Thu May 11 2017 sflees@suse.de
- Proper fixes for the last change re generic_server,
  * console should require enhanced_base so its functionally the
    same as x11
  * base should require minimal_base
  * remove the following from base again as they are in minimal base
  * openSUSE-build-key
  * openSUSE-release
  * rpm
  * other minor tidyup's
* Wed May 10 2017 dleuenberger@suse.com
- Re-introduce generic_server pattern: it requires minimal_base and
  basesystem (including zypper) and recommends yast2_basis. The
  use of the 'pure' console pattern did not work out as expected:
  none of the management tools are part of the generic console
  toolset (which is correct).
* Wed May  3 2017 kukuk@suse.de
- Add system users and groups to minimal_base
* Fri Apr 28 2017 jengelh@inai.de
- Compact %%install routine; replace old RPM shell vars by macros.
* Fri Apr 28 2017 dleuenberger@suse.com
- Do not require autoyast2 by base pattern.
* Fri Apr 28 2017 dleuenberger@suse.com
- Do not install salt-minion on every machine possible. This does
  belong to machines that are managed by salt, which is for larger
  setups. 'Common' installs won't need this.
* Wed Mar 15 2017 sflees@suse.de
- Create new base split package from old package
- Macros split into patterns-rpm-macros package
* Mon Mar  6 2017 sflees@suse.de
- Implement changes for the pattern split without actually
  splitting
  * Remove the following patterns.
  * devel_ide
  * devel_qt4
  * misc_server
  * remote_desktop
  * voip
  * rest_dvd9
  * rest_promo_dvd
  * generic_server (merged into console)
  * tabletpc
  * Minimal-Base, Base and Enhanced Base refactored to be closer
    to how they appear in SLE.
  * command-not-found and susepaste added to console and x11
    patterns as recmmends, this was requested by those providing
    opensuse support.
* Sun Mar  5 2017 sor.alexei@meowr.ru
- Require dbus(org.freedesktop.Notifications) in mate_basis: a
  notification daemon is a must.
- Recommend mate-notification-daemon in mate_basis as a preferred
  option.
- Suggest linphone instead of ekiga in mate_internet.
* Sun Feb 26 2017 fabian@ritter-vogt.de
- Replace kwrite with kate
* Sat Feb 11 2017 fabian@ritter-vogt.de
- Replace mozilla-kde4-integration with kmozillahelper
* Mon Feb  6 2017 dleuenberger@suse.com
- Bump version to 20170206.
* Fri Feb  3 2017 zaitor@opensuse.org
- Replace liberation-fonts with liberation2-fonts Recommends: With
  the current freetype2 we have in Tumbleweed, liberation2-fonts is
  more visually pleasing.
* Thu Feb  2 2017 dimstar@opensuse.org
- Drop devel_java pattern from the DVD: space constrains call for
  this action. The pattern of course stays available in the repos.
* Wed Jan 25 2017 tiwai@suse.de
- Drop gsynaptics Recommends from XFCE and LXDE patterns
  (boo#1010202):
  This drags xf86-input-synaptics and blocks the libinput usage as
  a consequence although we've already wanted to move to libinput.
* Tue Jan 10 2017 zaitor@opensuse.org
- Drop gnome-nettool Recommends from patterns-openSUSE-gnome and
  patterns-openSUSE-gnome_internet. Gnome-nettool is deprecated
  upstream and is no longer a package we want to install by
  default.
* Tue Jan 10 2017 fabian@ritter-vogt.de
- Remove konqueror references from patterns, one browser in the
  default installation is enough
* Tue Jan  3 2017 rbrown@suse.de
- Remove non_oss from the recommends of GNOME, KDE & MATE patterns
* Fri Dec  9 2016 sflees@suse.de
- Major cosmetic cleanup, No functional changes. Changes found by
  running zypper --whatprovides on each name listed in the pattern
  then fixing the differences.
  * Update packages that have been renamed
  * Drop packages that have been absorbed into other packages
* Fri Nov 18 2016 astieger@suse.com
- Fix spelling of Enlightenment boo#1010967
* Wed Oct 26 2016 alarrosa@suse.com
- Recommend kdeconnect-kde in the kde_plasma pattern. This plasma
  widget adds a lot to the user experience.
* Wed Oct 26 2016 zaitor@opensuse.org
- Drop aisleriot from recommended packages. App still depends on
  gconf, and we do not want to pull in gconf just for a game by
  default.
* Thu Oct 20 2016 dimstar@opensuse.org
- The DVD is exploding in size:
  + Drop devel_qt4 pattern from the DVD; the world has moved to
    Qt5. The pattern stays available in the online repos.
  + Drop the devel_gnome pattern: serious GNOME devs won't be using
    this from the DVD: most rely on jhbuild or at least on the
    content of the online repo.
* Tue Oct 18 2016 mvetter@suse.com
- boo#1005324: adding pavucontrol-qt without pulseaudio does not
  make sense.
* Wed Oct 12 2016 dimstar@opensuse.org
- Do not recommend tracker, tracker-gui and brasero for the XFCE
  Desktop: Tracker is well integrated into the GNOME Desktop and
  GNOME Applications, but XFCE itself can't do much with its
  database (boo#1002454).
* Fri Oct  7 2016 mvetter@suse.com
- Update LXQt pattern:
  * New core packages: lxqt-l10n, lxqt-sudo
  * Non core packages who still belong to LXQt upstream organization:
  obconf-qt, pavucontrol-qt
  * Replace gwenview by lximage-qt.
* Thu Sep 22 2016 dimstar@opensuse.org
- Update GNOME pattern to be more in line with the GNOME Core
  application recommendations:
  + brasero: only suggested instead of recommended. Writing optical
    media is no longer the 'standard'. By suggesting it, we ensure
    the programs stays on the media though.
  + gnome-clocks and gnome-logs: newly recommended tools.
* Thu Sep 22 2016 dimstar@opensuse.org
- Demote libreoffice-base-extenstions from Recommends to Suggests:
  we already don't explicitly recommend Base (which also does not
  make much sense for most users) - so the extensions make even
  less sense. Keeping it as suggests ensures it still ends up on
  the media though.
* Fri Sep 16 2016 alarrosa@suse.com
- Added kdebase4-workspace-libs Recommend to the kde_plasma pattern
  so qt4 apps use the breeze style instead of depending on indirect
  requirements to install that package.
* Mon Sep  5 2016 zaitor@opensuse.org
- Drop gnome-web-photo Recommends from gnome_utilities pattern.
* Thu Aug 25 2016 badshah400@gmail.com
- Suggest instead of Recommend inkscape for the gnome_imaging_opt
  pattern, so that it is not pulled in by default on default GNOME
  pattern installs. Inkscape is a specialised app that users
  should install on their system if they need it, and the other
  desktop patterns (e.g. KDE) do not install it by default either.
* Thu Aug 25 2016 dimstar@opensuse.org
- Add the 'generic_server' pattern to the DVD.
* Mon Aug 22 2016 jengelh@inai.de
- Drop sox, vorbis-tools from console, this does not feel like the
  right place for them. Same for wodim.
* Tue Aug  9 2016 dimstar@opensuse.org
- No longer recommend gnome-games-recommended by gnome_games but
  the actual games directly: iagno, lightsoff, quadrapassel,
  gnome-chess, gnome-mahjongg, gnome-mines, gnome-sudoku,
  swell-foop. The resulting installation pattern is the same.
* Mon Aug  8 2016 lnussel@suse.de
- remove references to rsh, smpppd, checkinstall. The packages got
  dropped long ago.
* Thu Jul 28 2016 dimstar@opensuse.org
- Use lightdm instead of lxdm for LXDE.
* Thu Jul 14 2016 fvogt@suse.com
- Add yast2-qt-branding-openSUSE to rest_cd_* patterns to
  get the default look of the installation wizard
* Mon Jul  4 2016 dimstar@opensuse.org
- Drop kstars from kde_edutainment pattern: we need to gain some
  space on the DVD again.
* Fri Jul  1 2016 tchvatal@suse.com
- Drop itrace, we don't have it on media anymore
* Sat Jun 18 2016 dimstar@opensuse.org
- Updates to pattern rest_cd_x11 (Rescue CD):
  + Always use kernel-default (never kernel-desktop): The kernel
    configurations have been merged and kernel-default provides
    all features. Same for ndiswrapper-kmp-default
  + Drop useless recommends, that would only matter if the rescue
    CD were installable: cracklib-dict-full, pullin-fluendo-mp3,
  + Replace mupdf with evince (33MB vs 3.5 MB, the GTK3 stack is
    already there anyway).
* Mon Jun 13 2016 mvetter@suse.com
- Add LightDM to LXQt pattern
- Require LXQt core components for LXQt pattern instead of only recommending them
* Fri Jun 10 2016 sflees@suse.de
- Add LightDM to enlightenment pattern (boo#951558)
* Mon Jun  6 2016 mvetter@suse.com
- Add a note about obconf-qt in lxqt pattern.
- Add qterminal recommends to lxqt pattern.
* Mon May 30 2016 tchvatal@suse.com
- Remove comments that do not make sense
- Remove pm-utils -> obsoleted by systemd
- Remove tmpwatch -> now handled by systemd
- Do not suggest and recommend elilo -> we have grub2 only supported
- Do not recommend gfxboot -> we have grub2
* Fri May 27 2016 dimstar@opensuse.org
- Drop icedtea-web recommends from gnome and kde patterns.
* Thu May 26 2016 dimstar@opensuse.org
- No longer recommend xorg-x11-Xvnc by the x11 pattern
  (boo#981663).
* Wed May 18 2016 mimi.vx@gmail.com
- remove lilo and grub from patterns
* Tue May 17 2016 dimstar@opensuse.org
- Add unbound to the DVD patterns (boo#972499).
* Fri May 13 2016 pgajdos@suse.com
- add devel_osc_build pattern
* Thu Mar 31 2016 dmueller@suse.com
- enable dmidecode/numactl/efibootmgr for aarch64
* Sun Mar 27 2016 badshah400@gmail.com
- Update GNOME patterns:
  + Remove gnome-font-viewer from all GNOME patterns (still leave
    it in the mate-* patterns).
  + rest_cd_gnome:
  - Drop gnome_games pattern to save space, add back a few
    games as Recommends instead (we don't want a completely
    game free LiveCD after all!).
  - Drop mc.
  - Drop backup and rescue utilities: ddrescue, gnu_ddrescue,
    grsync, lftp.
  - Drop krb5-client and pam_krb5.
  - Drop gparted, more suited to the rescue cd than here.
  - Drop ldap utilities: pam_ldap, nss_ldap.
- All patterns: Drop input-utils, package has been long dropped
  from Factory.
* Mon Mar 21 2016 tchvatal@suse.com
- Replace xchat by hexchat everywhere
* Thu Mar  3 2016 tchvatal@suse.com
- Fix pattern name/summary for python3 developement
- Fix pattern for KF5 with wrong python3-kde3 that never existed
- Add back tlp to laptop pattern which was dropped by previous changeset
* Wed Jan 27 2016 tittiatcoke@gmail.com
- Obsolete the kde4bindings for perl, python, ruby and mono. They
  haven't been ported and are obstructing the switch of Akonadi to
  its Frameworks port
* Sun Jan 24 2016 dimstar@opensuse.org
- Do not require midori on the Rescue CD: we already have Firefox
  on it; midori pulling in webkit makes it just too big.
* Sat Jan 23 2016 dleuenberger@suse.com
- Replace cdrecord in console pattern again with wodim: as long as
  kiwi hard requires wodim, we can't require cdrecord from the
  pattern.
* Thu Jan 14 2016 dimstar@opensuse.org
- Drop deja-dup from GNOME patterns: it was added back in the days
  when we increased the live CD from 650MB to 1000MB in order to
  help fill up the space - but now we are struggling to keep below
  the 1000MB limit.
- Drop tango-icon-theme from the GNOME base pattern: it's been a
  while that GNOME relied on anything but adwaita and hicolor.
* Tue Jan 12 2016 dimstar@opensuse.org
- Recommend pinentry-gnome3 from the GNOME pattern (boo#961472).
* Sun Jan 10 2016 tittiatcoke@gmail.com
- Adjust the KDE/PLASMA patterns:
  * Move to the new KF5 based KDEPIM suite as default
  * Move to kf5 based gwenview
  * Also recommend plasma5-pa for the kde_multimedia pattern instead
    of kmix
  * drop all kdeartwork related packages. This has been dropped
    completely upstream
  * minor corrections
* Fri Jan  1 2016 davejplater@gmail.com
- Replaced wodim with well maintained cdrecord in console pattern.
* Fri Dec 18 2015 mkubecek@suse.cz
- kde_plasma: hard dependencies have been added recently for
  packages that either are far from necessary or have reasonable
  alternatives some users may prefer; soften these to Recommends:
  + plasma5-pk-updates
  + plasma5-pa
  + dragonplayer
* Sat Nov 21 2015 badshah400@gmail.com
- Update GNOME patterns:
  + gnome_imaging_opt: Drop xsane, GNOME already includes
    simple-scan serving the same purpose.
  + gnome_internet:
  - Drop gftp, gwibber, smuxi, xchat-gnome.
  - Replace pan by evolution-rss.
  + gnome_utilities: Drop conduit, kerneloops-applet, pdfmod,
    synapse, tasque and the-board.
* Wed Nov 18 2015 tittiatcoke@gmail.com
- Remove kdeartwork4-screensavers from the kde-utilities pattern.
  This saves space on the LiveCD and is useless on a Plasma5 desktop
* Mon Nov 16 2015 dimstar@opensuse.org
- Drop valgrind-devel from devel_kde pattern: it is not very common
  to require that by devs.
* Mon Nov 16 2015 dimstar@opensuse.org
- Update GNOME patterns:
  + Replace shotwell with gnome-photos.
  + Drop alacarte: it makes only very limited sense since GNOME 3.
  + Drop system-config-printer-applet: GNOME 3 does not have a
    try to properly display it.
* Thu Nov 12 2015 coolo@suse.com
- bump version
* Wed Nov 11 2015 pgajdos@suse.com
- readd dejavu-fonts on coolo's request (comment in request#343080)
* Wed Nov 11 2015 mlatimer@suse.com
- Remove kernel-xen (and omnibook-kmp-xen) from xen_server pattern
  as kernel-default now includes pvops capabilities.
* Thu Nov  5 2015 pgajdos@suse.com
- install google-roboto-fonts instead of dejavu-fonts in -fonts
  pattern, add noto-sans to -fonts_opt pattern [bnc#951898]
* Mon Nov  2 2015 mlin@suse.com
- According to icewm changes https://build.opensuse.org/request/show/342069
  use icewm-default as recommends instead of icewm-lite.
* Thu Oct 29 2015 mimi.vx@gmail.com
- Remove flash from patterns
* Fri Oct 23 2015 sor.alexei@meowr.ru
- Hard-require very basic parts of MATE in mate_basis pattern.
- Recommend mate-tweak and hexchat (instead of xchat) in mate pattern.
* Wed Oct 21 2015 hrvoje.senjan@gmail.com
- kde_plasma now require plasma5-pk-updates package
- Remove apper recommends in kde pattern
* Tue Oct 20 2015 hrvoje.senjan@gmail.com
- Add recommends for baloo5-tools and baloo5-kioslaves to kde_plasma
  pattern (boo#948810)
- Given that video playback with vlc on Qt 5.5 is broken ATM, default
  to dragonplayer in plasma
* Sun Oct 18 2015 hrvoje.senjan@gmail.com
- Adjust kde plasma pattern to current state of things: require
  plasma5-pa instead of kmix
- Let plasma pattern be more useful: require at least one video
  player (vlc-qt)
* Thu Oct 15 2015 mlatimer@suse.com
- Suggest libvirt-daemon-lxc for both Xen and KVM patterns.
  [bsc#947242]
* Fri Oct  9 2015 schwab@suse.de
- Recommend grub2-arm64-efi on aarch64
- create_macros: use %%global instead of %%define
* Thu Oct  8 2015 coolo@suse.com
- drag in updatetest packages if with_betatest is defined in the project
  (Leap for now)
* Thu Oct  1 2015 bwiedemann@suse.com
- Add generic_server pattern
* Thu Sep 24 2015 dimstar@opensuse.org
- Recommend systemd-32bit in the rest_dvd_core pattern.
* Wed Sep 23 2015 dimstar@opensuse.org
- Also fix the KDE CD: sssd-32bit is recommended there too, so also
  recommend systemd-32bit.
* Wed Sep 23 2015 dimstar@opensuse.org
- When recommending sssd-32bit, also recommend systemd-32bit: the
  systemd-pam modules are needed in this case too (not strictly a
  requirement, but pam-config -a --systemd fails on a 64bit system
  when pam-32bit is installed but the 32bit pam modules are not).
* Tue Sep 22 2015 mvetter@suse.com
- Add basic lxqt pattern
* Fri Sep 18 2015 p.drouand@gmail.com
- create mate_internet and mate_office_opt patterns
- Recommend newly created patterns instead of GNOME's ones; recommending
  GNOME's intall plenty of packages unwanted by regular users
* Tue Sep  8 2015 pgajdos@suse.com
- added stix-fonts and ctan-latinmodern-fonts to fonts_opt
  [bnc#944629]
* Mon Sep  7 2015 hguo@suse.com
- Introduce four recently developed Yast modules into yast2 pattern
* Fri Aug 28 2015 trcs@gmx.com
- Do not recommend lxpanel-plugin-power-manager in %%package lxde:
  dropped upstream.
  https://mail.xfce.org/pipermail/xfce-announce/2015-May/000423.html
* Fri Aug 28 2015 mimi.vx@gmail.com
- tweak haskell_platform pattern
* Thu Aug 27 2015 lnussel@suse.de
- need to allow binutils in minimal
* Tue Aug  4 2015 mimi.vx@gmail.com
- add haskell_platform pattern
* Sun Jul 19 2015 coolo@suse.com
- downgrade broken requires from leechcraft pattern
* Thu Jul 16 2015 mlin@suse.com
- drop out dhcpcd
* Wed Jul 15 2015 sor.alexei@meowr.ru
- Adapt MATE patterns to MATE 1.10+.
* Tue Jul 14 2015 arnaud@versini.eu
- Recommend OpenJDK 8 instead of OpenJDK 7.
* Mon Jun 29 2015 dimstar@opensuse.org
- Recommend NTP at least until boo#936378 is fixed and YaST is not
  trying to configure a service that's not there.
* Thu Jun 25 2015 lnussel@suse.de
- remove minimal conflict with perl, perl-Bootloader-YAML draws it in via
  perl-YAML-LibYAML
- since we have perl now anyways recommending vim is no problem
  anymore either
* Tue Jun 23 2015 dimstar@opensuse.org
- Remove kdebase4-wallpapers from the kde pattern.
* Tue Jun 23 2015 dimstar@opensuse.org
- Remove gnucash and gnucash-docs from the patterns due to space
  constraints on the DVD media.
* Mon Jun 22 2015 dimstar@opensuse.org
- Drop gnome-do and hamster-applet from gnome_utilities pattern.
* Thu Jun 11 2015 lnussel@suse.de
- tweak minimal pattern to make it smaller again:
  * omit kernel. In theory YaST will add it for real installations.
  * don't install yast anymore. Should work in theory
  * remove adjtimex, hopefully not used anymore
  * remove SuSEfirewall2, needs full perl which is huge
  * remove eject, ntfs-3g and ntp, sysfsutils not really needed
  * remove release-notes-openSUSE, pulls in perl
  * remove vim, needs full perl :-(
  * require zypper instead of software stack pattern to avoid additional recommends
  * blacklist perl, python-base and binutils to avoid recommended packages
    pulling them in
* Wed Jun  3 2015 tittiatcoke@gmail.com
- Specifically obsolete the kdebase4-workspace-devel package
  by the kde_plasma pattern to prevent issues during an upgrade
* Tue May  5 2015 simon@simotek.net
- e17 pattern is now the enlightenment pattern and installs the
  latest enlightenment release rather then e17.
- The enlightenment pattern now recommends the following useful
  applications
    Mozzila-Firefox
    desktop-data-openSUSE
    xdg-user-dir
    clementine
    vlc
    leafpad
    mupdf
* Thu Apr 30 2015 tittiatcoke@gmail.com
- Add kdepim apps to the kde patterns.
* Tue Apr 28 2015 tittiatcoke@gmail.com
- Fix some kde4 left over patterns
* Mon Apr 27 2015 tittiatcoke@gmail.com
- The right package name is breeze and not breeze5 :(
* Fri Apr 24 2015 tittiatcoke@gmail.com
- Correction of the KDE related patterns
  * Ensure the right ones are required
  * Renamed devel_frameworks to devel_kde_frameworks
* Thu Apr 23 2015 tittiatcoke@gmail.com
- Update of all KDE related patterns
  * Plasma 5 desktop is the new default
  * _kde4_ patterns renamed to _kde_
  * Dropped the obsolete patterns for plasma5
  * A new devel patterns was added to pull in the KDE Frameworks
    development packages.
* Tue Apr 21 2015 dimstar@opensuse.org
- Drop gstreamer-0_10 dependencies: this has not been maintained
  for a couple years - if packages still rely on it, they are
  required to ensure the presence on their own. No more help from
  the patterns.
* Wed Apr  8 2015 crrodriguez@opensuse.org
- Bump version number.
* Wed Apr  8 2015 crrodriguez@opensuse.org
- Do not make "bootcycle" part of any pattern, it has to
  be either dropped or fixed to work with grub2.
* Sun Mar 29 2015 trcs@gmx.com
- Add some changes in LXDE's pattern to lighten the desktop, reduce
  the dependence on other desktops and improve user experience
  * Removed duplicated Recommends: lxdm line
  * Removed Suggests: moonlight-plugin
  * Removed suggestion of libreoffice-gnome if arch is x86_64
  * Recommend Atril instead of MuPDF
  * Recommend HexChat instead of XChat
  * Recommend lxpanel-plugin-power-manager
  * Recommend Pidgin instead of Empathy
  * Recommend pk-update-icon
* Sat Mar 28 2015 gber@opensuse.org
- Replace rhythmbox with pragha as the default audio player for
  Xfce, see
  http://lists.opensuse.org/opensuse-xfce/2014-08/msg00000.html
* Sat Mar 14 2015 javier@opensuse.org
- Replace tomcat6 with tomcat
* Sat Feb  7 2015 hrvoje.senjan@gmail.com
- Add some new packages to plasma 5 pattern: konsole5, kwrite5,
  kmix5 (boo#907604), kcm-touchpad5
* Mon Jan  5 2015 lslezak@suse.cz
- Replace subversion by git in the devel_yast pattern (bnc#911635)
* Wed Dec 17 2014 dimstar@opensuse.org
- gnome-icon-theme is deprecated in favor of adwaita-icon-theme
  (which we don't need to put in a pattern, as it's a dependency to
  a complete GTK+ 3.0 installation).
* Wed Dec 17 2014 mpluskal@suse.com
- Replace microcode_ctl with ucode-amd and ucode-intel
* Tue Dec  9 2014 dimstar@opensuse.org
- Reorder patterns in the Desktop Environment group: GNOME. KDE4,
  Plasma5, XFCE, LXDE, Enlightment, MATE, Fonts, X Windows System.
* Sun Nov 30 2014 p.drouand@gmail.com
- Prefer gparted to gnome-disk-utility by default in Mate environment
* Wed Nov 26 2014 coolo@suse.com
- let's remove some devel patterns from DVD
* Sat Nov 22 2014 p.drouand@gmail.com
- Add mate-polkit to mate-basis pattern; fix boo#905679
* Sat Nov  8 2014 p.drouand@gmail.com
- Add gnome-main-menu to mate_basis pattern; fix boo#904096
* Wed Nov  5 2014 gber@opensuse.org
- prefer the YaST Qt frontend in Xfce and LXDE as well since
  libyui-gtk and libyui-gtk-pkg are unmaintained and have been
  removed
* Fri Oct 17 2014 dimstar@opensuse.org
- Add adobe-source* fonts to fonts_opt pattern (boo#901756).
* Tue Oct 14 2014 badshah400@gmail.com
- Pattern sw_management_gnome: Add back gnome-software as
  recommended package
- Pattern gnome_internet: Add NetworkManager-openconnect-gnome as
  recommended package (boo#900009).
* Mon Oct 13 2014 badshah400@gmail.com
- Pattern gnome_utilities: Add gnome-weather as recommended
  package.
* Sun Oct 12 2014 badshah400@gmail.com
- Pattern gnome_internet: Replace xchat by polari
- Pattern gnome_multimedia: Remove rhythmbox as a recommended
  application, gnome-music will now be the default music player.
* Wed Oct  8 2014 p.drouand@gmail.com
- Recommends mate-search-tool instead of tracker, as tracker
  is not integrated to Mate applications for the moment
* Tue Oct  7 2014 opensuse@cboltz.de
- add apparmor-abstractions to AppArmor pattern
* Tue Oct  7 2014 coolo@suse.com
- bump the version to 20141007
* Tue Oct  7 2014 dimstar@opensuse.org
- Remove gnome-software for now: it triggers polkit requests for
  passing the proxy between the user session and packagekitd.
* Mon Oct  6 2014 p.drouand@gmail.com
- Add Mate patterns (boo#899665)
- Add translations in create_macros
* Mon Oct  6 2014 badshah400@gmail.com
- Patterns enhanced_base and minimal_base: Drop linux32 as a
  recommended package.
* Sat Oct  4 2014 badshah400@gmail.com
- Pattern sw_management_gnome: Add gnome-software as recommends.
* Thu Oct  2 2014 badshah400@gmail.com
- Pattern gnome_yast: Replace libyui-gtk-pkg and
  yast2-control-center-gnome in gnome_yast pattern with its qt
  version; the gtk bindings have critical bugs (see bnc#897910,
  bnc#891183, bnc#852730) but seem unmaintained for now.
* Sat Sep 27 2014 trcs@gmx.com
- package lxde
  * do not recommend imaging pattern:
    + it triggers the installation of GIMP that is redundant
    because mtPaint is in the pattern, has the same goal and is
    lighter
    + it recommends the installation of the pattern imaging_opt
    which in turn suggests calibre that is a heavyweight
    application that doesn't really fit in LXDE
  * do not recommend neither lxshortcut nor lxsession-edit:
    + lxshortcut is deprecated by LibFM 1.2.0
    + lxsession-edit is now provided by LXSession
  * recommend uGet instead of transmission-gtk and do not suggests
    aria2:
    + uGet can manage both BitTorrent and Metalink protocols (in
    addition to HTTP, HTTPS and FTP)
    + aria2 is required by uGet
* Sat Sep 13 2014 dap.darkness@gmail.com
- Added new leechcraft plugins; deleted p100q (obsoleted).
* Mon Sep  8 2014 coolo@suse.com
- replace pm-profiler with tuned
* Fri Sep  5 2014 dvlaeev@suse.com
- Recommend grub2-powerpc-ieee1275 on PowerPC instead of lilo
- Recommend haveged on ppc64
* Mon Aug 25 2014 lnussel@suse.de
- add iproute2 to minimal pattern
* Thu Aug 21 2014 gber@opensuse.org
- xfce_laptop: remove xfce4-panel-plugin-brightness which has been
  merged into xfce4-panel-plugin-power-manager and is installed
  through xfce4-panel-branding-openSUSE
* Wed Aug 20 2014 trcs@gmx.com
- package lxde
  * recommend gcolor2, currently one have to open a graphics
    program just to pick a color
* Wed Aug 20 2014 msrb@suse.com
- Replace tightvnc with tigervnc as default VNC client.
  (bnc#879917)
* Sat Aug 16 2014 hrvoje.senjan@gmail.com
- Add a Plasma 5 pattern. Mostly a copy of kde4_basis, with exceptions
  where needed
- Update kde_devel pattern: no longer recommend gstreamer_0.10, but
  standard one
* Sun Aug  3 2014 trcs@gmx.com
- package lxde
  * move Leafpad from Suggests to Recommends to replace Beaver because
    Beaver is buggy, isn't translated and this is not going to change
    as it is unmaintained too
  * recommend Geany as advanced text editor, it is light enough to fit
    in LXDE
  * recommend xfce4-screenshooter, mtPaint lacks a lot of features
    in this area
* Wed Jul 23 2014 coolo@suse.com
- translate C++ comments into spec files</brownbag>
* Tue Jun 24 2014 coolo@suse.com
- move ca-certicates-mozilla into base system
- switch to a more generic version scheme
* Thu May 29 2014 hrvoje.senjan@gmail.com
- Add Qt5 devel pattern
* Thu May 15 2014 gber@opensuse.org
- gcalctool has been renamed to gnome-calculator
* Wed Apr 23 2014 hrvoje.senjan@gmail.com
- Adjust kde patterns to current status:
  * Drop no longer necessary libsoprano-devel and strigi-devel
  * Drop plasmoid-quickaccess, no longer in distro
  * Recommend baloo-file and baloo-pim
  * Swap kcm_gtk for kde-gtk-config
  * Swap koffice components to calligra counterparts
  * Swap gstreamer_0.10 backend and plugins for 1.0 versions
* Tue Apr 15 2014 coolo@suse.com
- suggests postfix *and* recommend postfix
* Mon Apr 14 2014 coolo@suse.com
- fix kernel requires for rest patterns
* Fri Mar 21 2014 coolo@suse.com
- kde4 pattern does no longer exist
* Mon Feb 24 2014 dimstar@opensuse.org
- gnome-games: recommend gnuchess instead of phalanx (bnc#864907).
* Wed Feb 19 2014 coolo@suse.com
- rename kde4 pattern kde - that is visible
* Mon Feb  3 2014 coolo@suse.com
- make visible patterns visible
* Fri Jan 31 2014 coolo@suse.com
- patterns are no longer maintained in git (well, they are for
  now but that's just till I'm certain the rpm only way works)
  fate#309385 (internal)
* Tue Jan 28 2014 coolo@suse.com
- Dinar Valeev (1):
    Don't require libspe and pdisk on ppc64le
  Ludwig Nussel (4):
    port to libsolv, part 1
    implement explain again
    remove yast2-x11 from default install again
    remove debug output
  Stephan Kulow (1):
    Merge pull request #62 from k0da/master
  Commit: 334f1f9b
* Tue Jan 14 2014 coolo@suse.com
- Stephan Kulow (1):
    switch to generic names of libyui
  Commit: 174890fa
* Fri Dec 20 2013 coolo@suse.com
- Cristian Rodríguez (1):
    prefer kmod-compat instead of module-init-tools
  Lukas Ocilka (1):
    Dropped yast2-backup and yast2-restore from YAST_BASIS (FATE#308682)
  Raymond Wooninck (2):
    Plasma-nm replaces NetworkManager-kde4
    plasma-nm replaces NetworkManager-kde4 and arora no longer exists
  Stephan Kulow (1):
    use the right texlive package (bnc#854509)
  Tomáš Chvátal (1):
    Preffer the systemd-logger (journald) instead of rsyslog. Future here we go.
  Commit: 2251e2ad
* Tue Dec 10 2013 werner@suse.de
- Package texlive is nearly empty as it includes only the required
  TeX File System, therefore require texlive-scheme-tetex as this
  package cause the installation of more than medium, but nowhere
  near full TeXLive packages (bnc#854509)
* Mon Nov  4 2013 coolo@suse.com
- Stephan Kulow (1):
    remove test updates and malloccheck
  Tomáš Chvátal (1):
    Use mlocate in patterns over findutils-locate
  Commit: 46d3ece3
* Mon Oct 28 2013 coolo@suse.com
- Stephan Kulow (1):
    libvirt should only be a suggested package to the patterns
  Commit: d6fb2bf6
* Wed Oct 16 2013 coolo@suse.com
- Commit: d94d126e
* Mon Oct  7 2013 coolo@suse.com
- Jim Fehlig (1):
    Add libvirt to KVM and Xen patterns
  Stephan Kulow (1):
    put the apparmor pattern on the cds too - default is default
  Commit: d94d126e
* Fri Sep 27 2013 coolo@suse.com
- Atri Bhattacharya (1):
    GNOME-Utilities: Replace tomboy by bijiben.
  Stephan Kulow (6):
    remove 32bit pattern (bnc#821978)
    recommend apparmor (bnc#724829)
    move udev-configure-printer to live isos (bnc#808014)
    no longer install xorg-x11 unless it's an update
  Vincent Untz (1):
    GNOME: Remove several old packages and add gnome-photos to suggests
  Commit: 4d1ad2eb
* Tue Sep 24 2013 coolo@suse.com
- update url - pointing to github
* Tue Sep 24 2013 coolo@suse.com
- Dominique Leuenberger (1):
    GNOME: Update patterns
  Juergen Weigert (1):
    Preliminary removal according to bnc#840026
  Commit: 02f72072
* Fri Sep  6 2013 coolo@suse.com
- Michael Catanzaro (7):
    gnome games: remove gbrainy
    gnome-desktop: replace Tomboy with Bijiben
    gnome-desktop: remove alternative-status-menu
    gnome-internet: drop epiphany-extensions
    gnome-multimedia: explicitly add missing gstreamer plugins
    gnome-desktop: add deja-dup
    devel c(++): move ddd to suggests
  plusky (1):
    Replace runlevel editor with service-manager
  Commit: 12199b5e
* Wed Sep  4 2013 coolo@suse.com
- Stephan Kulow (1):
    recommend desktop-branding for every desktop
  Commit: 7dc182ab
* Wed Aug 28 2013 coolo@suse.com
- Jim Fehlig (1):
    Add kvm_server pattern to DVD recommends
  Lukas Ocilka (1):
    Removed yast2-mouse from yast_basis pattern
  Stephan Kulow (2):
    remove yast2-irda from yast2
    make sure we can ignore some things for some live cds
  Commit: 6bed7f31
* Thu Aug  8 2013 coolo@suse.com
- Stephan Kulow (1):
    readd xorg-x11 to the default install (for now)
  Commit: e531eee7
* Tue Aug  6 2013 coolo@suse.com
- Stephan Kulow (1):
    also require some yast2* for kde and gnome livecd
  Atri (1):
    [REST-CD-X11] Add yast2-x11 to required packages.
  Commit: 8f1e89ab
* Fri Aug  2 2013 coolo@suse.com
- Stephan Kulow (1):
    switch to xorg-x11-essentials
  Commit: 5df80cef
* Fri Jul 26 2013 coolo@suse.com
- Christian Boltz (1):
    AppArmor pattern - don't install pam_apparmor when selecting the pattern
  Jim Fehlig (2):
    Remove xen-doc-pdf
    Add kvm_server pattern
  Stephan Kulow (3):
    replace old db-4_5 with db-devel
    trying to create a core set of installation medium
    add systemd-analysis to the enhanced base suggests
  Commit: 3a4da77c
* Sun Jun 16 2013 coolo@suse.com
- Stephan Kulow (1):
    remove limal-perl from required packages
  Commit: 9e30f6d5
* Sat Jun  8 2013 coolo@suse.com
- Jim Fehlig (1):
    Add libvirt-daemon-xen to the xen_server pattern
  Stephan Kulow (1):
    readd update-test pattern and aaa_base-malloccheck
  Commit: 27a398e5
* Wed May 15 2013 coolo@suse.com
- Commit: 029692f4
* Tue May 14 2013 coolo@suse.com
- Stephan Kulow (1):
    e17 does not recommend xfce :)
  Commit: 029692f4
* Tue May 14 2013 coolo@suse.com
- Stephan Kulow (5):
    add a e17 pattern
    do not put webyast on DVD, it's too often broken
    remove XGL patterns, compiz is pretty much dead
    add all xf86-video-* (picked by supplements)
    put scribus off the DVD, I need to save space ;(
  Commit: 6d7f28b9
* Tue May 14 2013 coolo@suse.com
- Atri Bhattacharya (2):
    Replace unmaintained epdfview by mupdf
    [LXDE-Office] Replace unmaintained epdfview by mupdf.
  Stephan Kulow (1):
    replaced libyui4 with libyui5
  Commit: e5df4756
* Mon Apr  8 2013 coolo@suse.com
- Andreas Jaeger (1):
    Add util-linux, adjtimex, time, which to BASIS
  Michael Catanzaro (4):
    gnome: remove hamster-applet
    gnome: update alternative status menu package name
    gnome: gcalctool is now gnome-calculator
    Remove xvkbd from laptop, add eekboard to tablet
  Stephan Kulow (1):
    grub2-efi is no more - replaced by e.g. grub2-i386-pc
  Commit: cd1a5224
* Sun Mar 10 2013 coolo@suse.com
- Atri (1):
    [REST-CD-X11] Add yast2-country for more keyboard layouts.
  Stephan Kulow (1):
    do not install hplip by default any longer (bnc#649280)
  Commit: e2fdb74e
* Wed Feb 27 2013 coolo@suse.com
  Do not rely on preprocessor, set __i386__ explicitly on %%ix86
* Tue Feb 26 2013 coolo@suse.com
- Stephan Kulow (2):
    suggest libyui backends for kde and gnome
    install shim by default
  Commit: b9244511
* Wed Feb 20 2013 coolo@suse.com
- Atri (1):
    Add xchat to REST-CD-X11.
  Raymond Wooninck (1):
    Include kde4-print-manager in the default KDE4-DESKTOP pattern
  Stephan Kulow (1):
    add osc to the patterns (as suggested)
  Commit: 4e9f3fa9
* Thu Feb 14 2013 coolo@suse.com
- Atri (7):
    Add useful packages & xfce pattern to REST-CD-X11.
    Add xfce pattern the right way
    Add file-roller to XFCE-MINIMAL; grsync, lftp, gnu_ddrescue to REST-CD.
    Replace Imagemagick on REST-CD-X11 by ristretto on XFCE-MINIMAL.
    Add xfce4-appfinder to XFCE-MINIMAL pattern.
    Drop XFCE-MINIMAL; XFCE-BASIS is now the minimal XFCE.
    Add ristretto to REST-CD-X11.
  Stephan Kulow (17):
    remove yast from x11 cd
    don't have X11 pattern recommend yast
    split off REST-CD a REST-CD-DESKTOP not to be put on X11 CD
    prefer lightdm over xdm
    add some yast modules for recovery
    move gtk2-branding-openSUSE away from X11 CD
    xfce requires xfce_basis
    remove xfce pattern again
    split xfce-basis into 2, one of them minimal for the rescue cd
    have yast2-control-center-gnome on rescue cd too
    make some space on the promo dvd
    need more space on promo dvd
    readd grub to the cds ;(
  Will Stephenson (1):
    Add kio_mtp
  Commit: c8c69f73
* Mon Feb 11 2013 coolo@suse.com
- Dominique Leuenberger (1):
    Replace banshee with rhythmbox... mono is in a rather odd stage... crashy... bnc#800237
  Commit: 440e44d3
* Thu Feb  7 2013 coolo@suse.com
- Stephan Kulow (2):
    remove some bits from the DVD
    do not recommend ConsoleKit
  Commit: 430571b2
* Mon Feb  4 2013 coolo@suse.com
- Stephan Kulow (2):
  no longer install grub1 by default
  install grub2-efi and syslinux by default
  Commit: aa39420d
* Fri Feb  1 2013 coolo@suse.com
- Stephan Kulow (1):
    we're back to nscd
  Commit: 37d7d67f
* Fri Jan 25 2013 coolo@suse.com
- Guido Berhoerster (1):
    Reduce dependencies on GNOME packages in XFCE
  Michal Hrusecky (1):
    Replacing MySQL with MariaDB in LAMP pattern
  Raymond Wooninck (1):
    Remove ConsoleKit-devel and add the two new kdegames development libraries
  Commit: 485bc121
* Wed Jan  9 2013 coolo@suse.com
- Ludwig Nussel (2):
    update minimal pattern
    add --no-recommends
  Commit: 06838603
* Mon Jan  7 2013 coolo@suse.com
- Guido Berhoerster (1):
    Terminal has been renamed to xfce4-terminal
  Ludwig Nussel (5):
    don't install any smtp daemon in minimal
    add glibc-locale to minimal
    remove xinetd, yp-tools and ypbind from minimal
    ncurses yast also needs ncurses packager
    prefer cracklib-dict-small in minimal
  Michael Catanzaro (1):
    Update openjdk packages
  Commit: 254debd2
* Mon Nov 12 2012 coolo@suse.com
- Stephan Kulow (1):
    ulimit is dropped
  Commit: d8b2c6b4
* Tue Nov  6 2012 coolo@suse.com
- Stephan Kulow (1):
    fix previous commit
  Commit: 982ea6db
* Tue Nov  6 2012 coolo@suse.com
- Stephan Kulow (1):
    with systemd there is no special requirement for syslog
  Commit: 93316782
* Sun Nov  4 2012 coolo@suse.com
- Stephan Kulow (1):
    switch to libpng15
  Commit: 0c70f9f1
* Sun Nov  4 2012 coolo@suse.com
- bump version
* Sun Nov  4 2012 coolo@suse.com
- Cristian Rodríguez (2):
    Move acpid to ENHANCED-BASIS-OPT
    Remove "smolt" from patterns
  Guido Berhoerster (2):
    xfce: Remove gnome-bluetooth which drags in too many GNOME dependencies
    xfce: Include xfce4-panel-plugin-weather again since it has been fixed
  Stephan Kulow (2):
    even make acpid suggested only
    small cleanups
  Commit: 0f4d50b9
* Mon Oct  1 2012 coolo@suse.com
- Stephan Kulow (1):
    yast- frontends are now libyui-*
  Commit: 0dff97c8
* Tue Sep 18 2012 coolo@suse.com
  Vincent Untz (2):
    gnome: Remove evolution-webcal, replace evolution-exchange with evo-ews
    gnome: Remove mention of telepathy-butterfly
  Commit: 8f63ad57
* Tue Sep 18 2012 coolo@suse.com
- Ludwig Nussel (4):
    minimal: add acpid
    stabilize sort by sorting by name if size is equal
    add branding-openSUSE to test case
    minimal: prefer postfix to avoid exim (bnc#778862)
  Stephan Kulow (2):
    do not require texinfo but makeinfo
    remove some packages no longer existant
  Vincent Untz (1):
    gnome, lxde: drop bug-buddy and gnome-pilot from patterns
  Commit: 89e8bf32
* Wed Jul 18 2012 coolo@suse.com
- Richard Brown (1):
    Added grub2-branding-openSUSE to ENHANCED-BASIS
- Stephan Kulow (2):
    Push most books off the DVD - just too large
    removing some things not to be on DVD
  Commit: 8d62c13c
* Thu Jul 12 2012 coolo@suse.com
- Stephan Kulow (1):
    bnc#756777 - xstroke is pretty useless
  Commit: a6fc9972
* Thu Jul 12 2012 coolo@suse.com
- Stephan Kulow (1):
    add hyper-v.rpm to install media
  Commit: 467963c9
* Wed Jul 11 2012 coolo@suse.com
- Stephan Kulow (1):
    adding a books pattern (bnc#703899)
  Commit: 062f8b4f
* Fri Jun 29 2012 coolo@suse.com
- Guido Berhoerster (2):
    add some low-level dependencies to XFCE-BASIS (bnc#768268)
    switch back to xfce4-mixer as the default mixer
  Commit: 5d7e8758
* Mon Jun 18 2012 coolo@suse.com
- Guido Berhoerster (1):
    strip down XFCE_BASIS to a bare-bones Xfce desktop with only core applications
  Marcus Meissner (1):
    added pdisk by default on powerpc
  Commit: c90d295f
* Wed Jun  6 2012 coolo@suse.com
- Stephan Kulow (1):
    fix webyast rename to end on DVD
    add provides and obsoletes to the old webyast patterns
  Commit: 8143bd44
* Mon Jun  4 2012 coolo@suse.com
- Ladislav Slezak (1):
    merged webyast-ws and webyast-ui patterns into one webyast pattern
  Commit: a51b366d
* Tue May 22 2012 coolo@suse.com
- Raymond Wooninck (1):
    Make plymouth the default for the bootsplash
  Commit: 46ac3850
* Fri May 18 2012 coolo@suse.com
  * Changed version and license
* Thu May  3 2012 coolo@suse.com
- Cristian Rodríguez (1):
    move irqbalance to ENHANCED-BASIS-OPT it serves
    no real purpose nowdays on machines without hundred of cpus.
  Guido Berhoerster (13):
    completely remove non-existent package lightdm-branding-openSUSE
    completely remove xfce4-panel-plugin-cddrive and xfce4-panel-plugin-places which are not in Factory
    add back thunar-volman-branding-openSUSE
    make xfce_basis visible again since this is not a problem with patterns
    remove gnome-bluetooth since it requires gnome-control-center (bnc#732689)
    adapt to recent packaging changes
    xfce-utils has been removed upstream
    correct the list of panel plugins
    xfce4-panel-plugin-datetime does not need to be installed by default
    added xfce4-panel-plugin-xkb since xkb layout switching is useful
    install gnome-keyring, it is enabled by default in xfce4-session
    clean up comments
    add tracker-gui, tracker is already dragged in by default
  Stephan Kulow (1):
    grub2 is needed now
  Commit: 772347e1
* Wed Apr  4 2012 coolo@suse.com
- Stephan Kulow (3):
    add more yast languages more to the CDs
    clean up requires of rpm build environment
    drop xgl/compiz from DVD
  Vincent Untz (3):
    gnome: Add nautilus-extension-seahorse and update for gnome-utils split
    gnome: Add gcr-viewer
    gnome: Add gsf-office-thumbnailer
  Commit: dd2a4380
* Tue Nov  8 2011 coolo@suse.com
- Stephan Kulow (2):
    add elilo and efibootmgr to the media (bnc#728529)
    hide xfce_basis for now (bnc#728319)
  Commit: e0f5a60b
* Fri Nov  4 2011 coolo@suse.com
- Stephan Kulow (1):
    revert bootsplash-branding change
  Commit: 76a0a769
* Fri Nov  4 2011 coolo@suse.com
- Stephan Kulow (2):
    any branding will do - openSUSE preferred
    take out update_test from default install
  Commit: c2de877b
* Wed Nov  2 2011 coolo@suse.com
- Stephan Kulow (2):
    we need to save 85mb off the dvd9
    don't install gnome-color-manager by default (bnc#698250)
  Commit: be94d305
* Mon Oct 31 2011 coolo@suse.com
- Richard Brown (1):
    gnome: Install gnome-shell-extension-alt-status-menu by default
  Stephan Kulow (1):
    suggests on promo dvd is just too much
  Commit: d595948d
* Fri Oct 28 2011 coolo@suse.com
- Stephan Kulow (2):
    make sure we require a kernel (bnc#674838)
    switch KDE updater to apper
  Commit: 9f3745a8
* Fri Oct 21 2011 coolo@suse.com
- Stephan Kulow (1):
    remove xstroke from laptop pattern, it's fine in tabletpc only
  Vincent Untz (1):
    gnome: Move simple-scan from GNOME-IMAGE-OPT to GNOME-IMAGE
  Commit: d4b44eba
* Fri Oct 21 2011 coolo@suse.com
- Ludwig Nussel (3):
    better warning if pattern lacks 'sel'
    minimal: suppress python and yast2-branding
    make test runner more generic
  Commit: 98a3c6ca
* Tue Oct 18 2011 coolo@suse.com
- Stephan Kulow (2):
    more renames
    readd ppds (bnc#652292)
  Commit: f6d66dda
* Fri Oct 14 2011 coolo@suse.com
- Stephan Kulow (5):
    remove rpmlib requires, those are really odd
    remove packages dropped for 12.1
    remove more packages not existant (anymore)
    remove some more stuff not present in factory
    follow renames and remerges
  Commit: e8af0ff0
* Fri Oct 14 2011 coolo@suse.com
- fix specfile generation
* Fri Oct 14 2011 coolo@suse.com
- Guido Berhoerster (23):
    drop opensuse-codecs-installer which is gone from Factory
    remove non-existent xfce4-branding-openSUSE and replace it with the real branding packages
    add pk-update-icon which replaces gpk-update-icon
    explicitly depend on system-config-printer-applet
    replace xarchiver with file-roller, xarchiver is dead and squeeze is not ready yet
    add back xfce4-panel-plugin-mixer as a possible alternative to gmixer
    replace xfburn with brasero, xfburn is unmaintained upstream and has numerous bugs
    add with rhythmbox a more sophisticated media player and sound-juicer to allow CD ripping
    add with simple-scan a more simple and usable alternative to xsane
    added a couple of applications for feature parity with the default installs of other desktops
    replace remote_desktop with remmina
    add some games for Xfce
    ease debugging by adding gdb
    no need for explicit xorg-x11 any more since we use lightdm
    add gnome-bluetooth to desktop pattern as well, bluetooth is also useful on desktop systems
    add alacarte, since Xfce 4.8 menus can be edited by it
    replace thunar-thumbnailers with tumbler
    neither the xfce not the murine gtk2 engine are needed since we switched from Sonar to Adwaita
    remove non existent packages
    remove xfce4-dict which is already recommended
    split off xfce_basis pattern in order to allow for more minimal installs
    added shotwell for photo management
    add xfce4-panel-plugin-datetime so it can be used by branding
  Ludwig Nussel (5):
    jarnal doesn't exist since 11.3
    conflict zypper-log in minimal due to python
    add klogd to BASIS
    recommend yast2-ntp-client in yast install pattern (bnc#723018)
    add script that works with perl-satsolver
  Stephan Kulow (2):
    add tons of packages useful when building KDE
    take out some of the largest packages
  Vincent Untz (1):
    gnome: Add gnome-control-center-branding-openSUSE
  Commit: f6407989
* Wed Oct 12 2011 coolo@suse.com
- Stephan Kulow (1):
    switch /sbin/init to systemd (bnc#723041)
  Commit: 7040c30e
* Thu Oct  6 2011 coolo@suse.com
- Stephan Kulow (1):
    flash-player seems to be 64bit now too
  Vincent Untz (4):
    gnome: Add gnome-documents to GNOME-DESKTOP
    gnome: Update for new name of package with dynamic background
    gnome: Pretty sure we don't need cups-backends by default
    gnome: Install gnome-color-manager, drop pulseaudio-esound-compat
  Commit: bc85b9a8
* Thu Sep 29 2011 coolo@suse.com
- Guido Berhoerster (1):
    switch from gdm to lightdm
  Commit: cb7af52a
* Tue Sep 20 2011 coolo@suse.com
- Andrea Florio (1):
    replace gpicview with viewnior
  Commit: 7cac039c
* Wed Sep 14 2011 coolo@suse.com
  Stephan Kulow (3):
    try promo dvd with suggests
    make systemd default on live cds
    do not install xgl on DVD either by default
  Vincent Untz (2):
    gnome: Rework some GNOME patterns
    gnome livecd: Do not install xgl/gnome_xgl patterns
* Mon Sep  5 2011 coolo@suse.com
- Ludwig Nussel (3):
    remomd converter
    better error message
    add yast2 added packages and locale=en
  Stephan Kulow (5):
    libatm1 is the new name (bnc#695303)
    GNOME3DONE define is old
    Ignore files
    put elilo on DVD
    put elilo and grub2 on DVD only so far
* Thu Sep  1 2011 coolo@suse.com
- Stephan Kulow (2):
    add libQtWebKit-devel to qt pattern (bnc#680081)
    adding siga to dvd (bnc#697047)
* Thu Sep  1 2011 coolo@suse.com
- Stephan Kulow (5):
    recommending minimal yast2 installation workflow for mininmal
    adding minimal_base-conflicts to the DVD patterns
    next round of minimal requires
    python-base is required through systemd now
    conflict with more supplements
* Mon Aug 29 2011 coolo@novell.com
  Ludwig Nussel (1):
    add zypper to minimal (bnc#713317)
  Stephan Kulow (1):
    add kiwi-media-requires for ms to track what should be on DVD
* Wed Aug 24 2011 coolo@novell.com
- Andrea Florio (1):
    opensuse-codecs-installer is obsolete
  Sascha Peilicke (1):
    Don't require/recommend AppArmor.
  Stephan Kulow (1):
    remove the default pattern, it has no meaning
* Tue Jul 26 2011 coolo@novell.com
- Andrea Florio (1):
    added simple-scan
  Stephan Kulow (1):
    readd update_test
  Vincent Untz (1):
    gnome: Add libgnomesu Recommends
* Tue Jul 12 2011 coolo@novell.com
- Stephan Kulow (1):
    recommend systemd for easier testing
  Vincent Untz (3):
    Update GNOME patterns for GNOME 3
    Do not require gnome-session-*-session, but recommend them
    gnome: Move tasque from Recommends to Suggests
* Tue Jun 28 2011 coolo@novell.com
- Stephan Kulow (4):
    add snapper to the yast patterns for better btfs support
    bootloaders are not strictly required in today's virtual world
    compiz is back
    webyast fits also on dvd9
* Fri Jun 17 2011 coolo@novell.com
  Stephan Kulow (1):
    fix webyast patterns and put them on dvd
* Wed Jun 15 2011 coolo@novell.com
  Stephan Kulow (2):
    drop ivman
    fix webyast sel files
* Wed Jun 15 2011 coolo@novell.com
- set version to 12.1
* Wed Jun 15 2011 coolo@novell.com
  Ludwig Nussel (2):
    add vim to minimal_base
    create a separate minimal_base-conflicts pattern
  Stefan Schubert (1):
    added webyast
  Stephan Kulow (4):
    libstorage is gone
    promo dvd can live without xgl
* Wed May 25 2011 coolo@novell.com
  16dc760 added missing YaST GTK GUI (bnc#688084)
  53df32b switch back to totem now that parole has dropped its browser plugin
  ab01071 replace mousepad with leafpad since the former will be dropped from Factory
  0e4f41e add perl-doc for devel-perl (bnc#688877)
  568ae31 take out problematic GNOME2 packages to get something rolling
  616567e remove compiz too for GNOME3DONE
* Wed Mar 30 2011 coolo@novell.com
  7cacb80 phonon-backend-xine is unused
  e9fc6d7 add texlive to technical writing (bnc#673657)
  d414e22 fix syntax
  b6cc0c4 we need both frontends
  3fdaf85 add sssd to the media (bnc#679439)
  288aad0 aria2 is not so common anymore
  ef23220 replace ksmolt with smolt-gui
  c295d0a licenses are no more
* Fri Feb 25 2011 coolo@novell.com
  394ae9c revert some empty files
  ce02fc4 add dependencies of skype
  b285e13 move skype deps out of default install
  0286502 add depencies of acroread to the DVD
  7cacb80 phonon-backend-xine is unused
* Mon Feb 14 2011 coolo@novell.com
  4973860 unrar is unfree now
  4f8c55d throw off some patterns from promo
  1a68b1f remove things from the biarch dvd
  6429034 take out update_test from default install
  7c526e2 take out empty files
* Mon Feb  7 2011 coolo@novell.com
- bump version
* Mon Feb  7 2011 coolo@novell.com
  ee7c122 move LanguageTool out of default install
* Mon Feb  7 2011 coolo@novell.com
  8d5e729 rest_dvd can't require two conflicting patterns
  38e535a gfxboot should be installed in !minimal
  0e2b4ef leave the full cracklib dict in minimal
* Tue Feb  1 2011 coolo@novell.com
  20e6ffa Change gtk theme to oxygen-gtk
  42c01f3 Recommend kdevelop4 for IDE pattern, suggest PHP
  4b543b4 Add Krita to Imaging pattern
  75516ab Add Rekonq to recommends for KDE Internet pattern
* Fri Jan 28 2011 coolo@novell.com
  659d680 remove xfce4-volstatus which will be dropped from Factory
  c74c68e hal-devel is no longer part of KDE
  afe82ea testing a minimal pattern
  d2c0b61 rename to minimal_base to avoid conflict with sles
  62592b4 conflict some bad recommends and supplements
  9207be2 a couple more conflicts for minimal
  6a04259 fix the name of the pattern package
  a63758d also conflict with cups-client
  dee5016 remove 2 packages of the top 10 installs
* Mon Jan 17 2011 coolo@novell.com
  8120148 do not recommend too much from base - base is the minimal
* Mon Jan 17 2011 coolo@novell.com
  bfa19cc follow virtualbox rename
  d4c5350 ksame is gone
* Tue Jan 11 2011 coolo@novell.com
  22d4883 removed corefonts from default install (bnc#655760)
* Mon Jan 10 2011 coolo@novell.com
  1a48484 remove no longer existant packages
  de6e0e7 cups-drivers is no more (bnc#652292)
  5ee174c changes from XFCE team
* Mon Dec 13 2010 coolo@novell.com
  220695b add Xfce patterns modifications by gber
* Tue Nov 23 2010 coolo@novell.com
  7c56627 disarm kde4_pure pattern
* Fri Nov 19 2010 coolo@novell.com
  75fd438 change to kpackagekit
  df7297f Use the NetworkManager plasmoid now
  fe73adb Install pulseaudio again by default
  b388743 ksshaskpass is broken and unmaintained
  c29ebd2 Kwebkitpart is a central part of 4.6
  f439f79 Correct Nepomuk requirements and move to Recommended
  4a84f97 BlueDevil replaces KBlueTooth
  a92ffb7 add haveged to optional base packages
  f70d1a1 make it possible to switch to syslog-ng
* Wed Nov  3 2010 coolo@novell.com
  f2a57ea Revert "remove update_test for release"
* Wed Nov  3 2010 coolo@novell.com
  e86af77 GNOME: Remove libraries from GNOME-BASIS
  cc3d493 GNOME: Add/remove some packages, and add many suggestions
  81cb448 BASIS: replace hal Recommends with pm-utils
  8549265 switch to libreoffice
  b6f2cf0 fix package names
  4f6573e tracked in bugzilla
  ee5d98f use cronie by default
* Tue Sep 28 2010 coolo@novell.com
  376de2b use empathy instead of pidgin
  6fb5b3b preload is not strictly required
  b9cc276 moblin pattern is gone as most packages of it are gone too
  e2068ef remove packages gone from the distribution
  3f5cdf7 Remove some obviously unneeded parts in GNOME patterns
  2bc03b8 Update GNOME-IDE to include devhelp and glade3
  19c6c60 Reorder GNOME-BASIS so that low-level bits are easily findable
  c21ed96 Remove yast bits from GNOME-BASIS
  a3141f8 Make packages in GNOME-Games recommended instead of required
  e6a80e0 Move packages around all the GNOME-* patterns
* Mon Aug 30 2010 coolo@novell.com
  02ade03 add some packages
  4ba8669 exim has no maintainer, not good to suggest
  7481311 adding quota-nfs to DVD
* Wed Jun 30 2010 coolo@novell.com
  be2a41e remove moblin-app-installer as the repos dont exist
* Tue Jun 29 2010 coolo@novell.com
  01877c3 add a dependency free GPG gui
  bce5ab3 we need hal's init script for cpufreq to work (bnc#612131)
* Mon Jun 21 2010 coolo@novell.com
  86b748c Do not recommend yast2-qt from GNOME-YaST
  4e0b698 added a touchpad configuration tool
  1c8c7f7 remove packages not installable
  67a91dc more cleanup
  6578467 replace capabilities with real package names
  3c7d755 one more provides
* Thu Jun 17 2010 coolo@novell.com
  c4fd58f complete the switch to yast2-gtk
  e4ee1f1 remove update_test for release
* Wed Jun  2 2010 coolo@novell.com
  a6a23b4 yast2-repair is gone
* Sun May 30 2010 coolo@novell.com
  bd0fb67 removed stuff vuntz thinks are obsolete
  4e20568 add _opt patterns to promo dvd
* Thu May 27 2010 coolo@novell.com
  f9e4ee6 revert fix for bnc#538897
  1f298ac Use tracker instead of beagle
* Thu May 27 2010 coolo@novell.com
  1136323 added a printer config tool
  88cd8e9 do not use PolicyKit anymore
  672566f remove non-building apps from selection
  e81c9ef too late to make it visible
  50d61f0 fixed order
  0f21c40 add libvdpau for bnc#605888
  9d65b15 small cleanup
  838e7a2 Drop outdated HAL from (enahnced) base system.
* Tue May 18 2010 coolo@novell.com
- remove dvd9 rest pattern
* Tue May 18 2010 coolo@novell.com
  b232af0 compcache is disabled
  900dc8f full up the promo dvd
  ec0c5b6 adding dvd9 specific pattern, only dummy "I want it all" for now
  1be5421 add things to the dvd9 that are normally only online
  0ff8f18 some more packages for dvd9
  2a5dfd5 added lxcc and improved lxde user experience
  517a312 added video player to LXDE
  402c6c2 use brasero instead of xfburn
  8e872de use GTK UI for YaST
  fbea081 use network manager by default
  25badf1 adapt to dropped yast modules
  721ffab back to sysvinit (http://lists.opensuse.org/opensuse-factory/2010-05/msg00168.html)
  97976e9 remove tmpwatch again
  a9a6ad9 remove all of Policykit but the -kde and -gnome frontends
  cca5a50 replace okteta with synaptics
  cba9a5f added moonlight-plugin
  61a351f Use empathy instead of pidgin by default for GNOME
* Wed Apr 14 2010 coolo@novell.com
  fc508de recommends office-lxde pattern
  7cf97b9 Default office patter require OO.o  Recommends lxde-office pattern instead
  0ad93d6 Revert "Switch to the new at-spi2 stack by default"
  e4a5bc8 add open-vm-tools (bnc#591085)
  984591e replace gnome-games (bnc#594503)
  4bbb001 add LanguageTool into office pattern (for dvd)
  9d6737e add tmpwatch to default (bnc#594778)
* Thu Apr  1 2010 coolo@novell.com
  4c3c16a moved claws to main LXDE pattern; removed polkit-gnome from pattern
  78a70a5 recommend the openSUSE branding for gnome
* Mon Mar 22 2010 coolo@novell.com
  19bce1a replace sysvinit with sysvinit-tools
* Mon Mar 22 2010 coolo@novell.com
  265bbb6 let's try with upstart for M4
  cf09420 add virtualbox-ose-guest-tools to liveCD
* Mon Mar  1 2010 coolo@novell.com
  87e55e0 add gnome utilities to the live cd
  53de3f2 replacing cscope with ctags in kernel development (bnc#582415)
  eb621b7 adding 32bit gtk to the DVD
* Sun Feb 14 2010 coolo@novell.com
  5bb8c74 improved lxde patterns, created lxde_laptop patter to provide some laptop utilities
  1c1b519 added lxde_laptop patter into dvd (no additional space required)
  0118b3f move sax2-gui from live cd to save space
* Wed Feb 10 2010 coolo@novell.com
- save the original spec file, so that autobuild does not
  remove the important comments ;(
* Mon Feb  8 2010 coolo@novell.com
  505b6b3 moving knode, kcron and ksystemlog out of the default install
  1987df9 add gnome-disk-utility (bnc#554954)
* Thu Jan 28 2010 coolo@novell.com
  26e7f97 put lxde on dvd - let's see how large it is
* Mon Jan 25 2010 coolo@novell.com
  93eff17 added lxde patterns
  1a1ca32 Switch to the new at-spi2 stack by default
* Thu Jan 21 2010 coolo@novell.com
  b9a7c66 add susehelp to the kde basics (bnc#541820)
  fdefc61 do not require but recommend opensuse branding (bnc#564730)
* Wed Jan 20 2010 coolo@novell.com
- update version to 11.3
  34c4417 move samba out of livecd patterns
  e56c82c adding meanwhile plugin (bnc#549711)
  b537a95 add xorg-x11-driver-video-unichrome to X11 suggestion list
  588d38c take out preload for now
  6346418 This reverts commit fb883332c54fbf0e5fc805c0cc779524221c1dda, readding update_test
* Fri Oct 30 2009 coolo@novell.com
  1383d75 no longer recommend xgl pattern for kde - it requires gnome (bnc#550875)
  34c4417 move samba out of livecd patterns
* Fri Oct 23 2009 coolo@novell.com
  fb88333 take down update_tests
  93f469e fix up ipw-firmware
  27aeba0 add kiwi-templates
  e80f252 do not require a flavor (bnc#547900)
  1065957 recommend kernel-syms for kernel development (bnc#548933)
* Tue Oct 13 2009 coolo@novell.com
  989b341 ggreeter is gone (bnc#543759)
  76f825b add samba to the default installation
  7468384 add alacarte to the livecd default too
  65f7a88 yast2-gtk-pkg is no more
* Fri Oct  2 2009 coolo@novell.com
  5c7650e require the patterns packages
  9dc8da1 games_opt is not needed
* Thu Oct  1 2009 coolo@novell.com
  2e37890 bnc#542936 - add webpin yast module
  84eae4d add firefox kde4 integration
  c1cae84 ufraw-gimp was renamed to gimp-ufraw
  b3c3106 cleanup dead and renamed packages
  4b7f5b9 trying with everything required
  63d6898 libusb-devel is now compat-devel
  c2e3b20 removed more dropped packages
  b62dcf9 another round of dropped packages
  16a84a3 done with dropped packages
* Mon Sep 28 2009 coolo@novell.com
  5cb7c95... add gwibber to DVD
  d7f6916... change from lsb to lsb-release by default (bnc#537378)
  e1cf9f1... adding smolt
  9a0bd24... adding btrfsprogs
  356ac46... adding gnome-packagekit to XFCE
  cc964c8... add lsof to live cd
  3da4dae... remove beagle-firefox from GNOME default (bnc#533783)
  846e43a... merge KDE4-BASIS-OPT back into KDE4-BASIS
  d6c64cb... qlogic-firmware is now kernel-firmware
  d96f273... move vpn plugins to the default install (bnc#533580)
  c23700e... leave out file conflicts with kernel-firmware for now
  cdf4380... try texlive on x86_64 again
  102879c... adding polkit-gnome (bnc#538897)
* Mon Sep  7 2009 coolo@novell.com
  d6de7da... xdm is not provided, hardcode xorg-x11
  3d4a20a... add droid fonts to default install
  d7bdcea... add specific icon themes for ooo
  1af1874... fix minimal autoyast installation
  69e0d1b... trying to install the update test by default
* Fri Aug 28 2009 coolo@novell.com
  05f6fb8... merge kde4_office_opt back into kde4_office only suggest the kde3 quickstarter
  87a6457... put the update_test on cds too (bnc#533154)
  36479d4... pull in kdebase4-openSUSE-pulseaudio-conflict
  4ba63a6... take out the pluseaudio block package, may exist better solution
  6699349... live without banshee for the moment
  d939c19... put more vpn on the dvd (bnc#533580)
  c724b4c... cups-autoconf should be replaced by udev-configure-printer (bnc#532838#10)
  b9702b4... xdm for xfce
* Thu Aug 20 2009 coolo@novell.com
  a99362e... patch from Beineri to put more kde apps into default selection
  20064f3... 3 PA tools no longer needed (bnc#529059)
  3c0a04d... monsoon is gone, transmission is default (bnc#530416)
  0d46f55... try compcache on the live cd
  c683570... harmonize the dvd default install with the live cd (patch by Binner)
  ec139ac... tex is too large for x86_64 (at least for now)
  affbf98... list polkit explicitly
* Tue Aug 18 2009 coolo@novell.com
  d4f0912... ksmolt on kde4
  7215e19... move exiftool out of default installation (bnc#529052)
  0fb8e24... move yast2-qt off the gnome live cd (bnc#529046)
  92ae5c0... NM should have much better support for umts now without using umtsmon (bnc#529050)
  06f971f... sumf is not maintained and not ported to qt4
  8414914... extra kernel is no more
  1e08776... sax is no longer required to have a working X
  670269c... put kinternet on the DVD for those that need it (bnc#527549)
  06b0f5a... no need to list kernel-*-base explicitly - especially when it will go away
  a91ceb7... remove some libraries
  4161f12... switch to xine backend for phonon
  96c9f5a... take out pulseaudio for testing
  dc8ff1e... move gstreamer plugins to gnome now that KDE uses xine backend
  4f16b18... recommend esound-daemon for KDE
  d21633f... if only kde4 is selected, it conflicts with pulseaudio
  f89c698... do not put laptop pattern on live cds, but take the network
* Tue Aug  4 2009 coolo@novell.com
  1be4e81... add pavucontrol for bnc#522801
  54f3fbb... removing gsynaptics for bnc#523500
  70bc3a4... make kde3 depdency suggested not required
  b95d000... put xdg-users-dir in default selection
  949873b... drop -pae and take -desktop on all archs
  48380d2... trying something:  switch nscd to unscd  switch syslog-ng to rsyslog
  8881eca... bug509829 - include xdg-user-dirs-gtk in GNOME
  06beee8... add kde internet stuff
  e7ff730... on 64bit require kernel-desktop
  7ecc5b9... only kernel-desktop on x86_64
  de09c63... patch from KDE team (bug#526987)
  2da8d35... take out xen_server for now and hide koffice2
  13bb9fd... kdebase3-devel is not needed anymore
* Tue Jul 21 2009 coolo@novell.com
  593cff4... add mc
  9eb74c6... bootchart is not interesting for most
  a7c9ecb... moving krecord to suggested and going from pinentry-qt ot -qt4
  2269c3a... replacing dropped kde3 apps (bnc#513833)
  d03eb1e... adding qt-creator
  3ddaf3b... moving command-not-found to live cd, fixing krecord
  a4dd766... adding yakuake
  3e8c4e5... bnc#521180
  e3d1df0... don't put libqt4 dependencies into enhanced-basis
  c5f2520... move X11 dependencies out of enhanced-basis
  98d34b9... suseRegister isn't required for suse
  adeeb40... we don't need no registration
* Tue May 26 2009 coolo@novell.com
  7329423... adding video drivers and server
  4009508... not that too many care for s390 opensuse, but it should stay without x11
  3f1da14... give it a requires too
  3573c91... fixing patterns
  57b4822... the package is back
  e541659... give me display
  3df2261... push changes from vuntz
  2c10c7f... ignore amarok for now
  522f40b... replace pae with legacy on the DVD
  0229227... xfce is still broken
  2587e91... readd it
  fe5c487... the patterns conflict, so don't require them
  e977ab1... Revert "- replace pae with legacy on the DVD"
  6c26bfa... readd amarok
* Tue May  5 2009 coolo@suse.de
- follow package names
* Wed Apr 22 2009 coolo@suse.de
- remove manuals pattern
* Tue Apr 21 2009 coolo@suse.de
- fix xfce pattern and clean up some left overs
* Tue Apr 21 2009 coolo@suse.de
- fix name of digikam
* Wed Apr 15 2009 coolo@suse.de
- update XFCE pattern (bnc#493549)
* Thu Apr  9 2009 coolo@suse.de
- remove all KDE3 patterns
- clean up some package names (follow renames)
* Tue Mar 31 2009 coolo@suse.de
  ca0d789... do not install ufraw by default (bnc#298969)
  86242bb... use kde4-konversation
  7e45fe1... hide kde3
  10a7a3b... go for kde4
  b90facf... adding wireless helpers
  2fb5e5f... no beagle on kde4
  bd2b801... leave out more of it for the time being
  47490c5... need these
  dc18426... trying to get new livecds
  1e33bbd... back to real life
  9d63d25... readd office
* Thu Feb  5 2009 coolo@suse.de
- adding python-devel to DEVEL-Python (bnc#456496)
- add more packages to devel-kde to allow building kde from svn
- require gdm for xfce
- checking who complains if zip is no longer installed
- daring something
  * drop compiz from livecd-kdee4
  * replace amarok with kde4-amarok
  * replace digikam with kde4-digikam
  * block OpenOffice-kde
  * adding umtsmon
- some ps3 moves
* Tue Dec  2 2008 coolo@suse.de
- office didn't make it for ppc (bnc#449711)
* Tue Dec  2 2008 coolo@suse.de
- fix typo
* Tue Dec  2 2008 coolo@suse.de
- require more of KDE3 explicitly
* Mon Dec  1 2008 coolo@suse.de
- do not recommend boost-doc - boost-devel eats for two
* Sun Nov 30 2008 coolo@suse.de
- require NM-kde to make the solver pick the "right" NM-client
* Sat Nov 29 2008 coolo@suse.de
- several workarounds for generic provides messing standard
  installs (bnc#450161)
* Fri Nov 28 2008 coolo@suse.de
- put the pullin-flash and pullin-mp3 also on livecd
- put the pullin-mswebfnt on all media
* Fri Nov 28 2008 coolo@suse.de
- vym is kdissert for KDE4 (says Uwe)
  - bnc#446088 - openjdk-plugin (and tons of others) not installed
  with kde 4.1
  - use openjdk-plugin explicitly and not generic "java-plugin"
  - fix syntax in MULTIMEDIA
  - make amarok1 default again (bnc#449422)
  - readding games to x86_64 cds (bnc#449832)
  - disable fbset (bnc#304995)
* Mon Nov 24 2008 coolo@suse.de
- add avahi explicitly to all desktops (bnc#446017)
* Mon Nov 24 2008 coolo@suse.de
- libusb-compat-devel -> libusb-devel (bnc#447572)
- kerneloops not installed by default for now
* Thu Nov 20 2008 coolo@suse.de
- back to NM-kde3
- move biosdevname out of the default install
* Mon Nov 10 2008 coolo@suse.de
- give NM-kde4 another beta
* Mon Nov 10 2008 coolo@suse.de
- do not autorequire the devel-tcl pattern
* Sun Nov  9 2008 coolo@suse.de
- split out the patterns only available on ftp (bnc#432508)
* Sun Nov  9 2008 coolo@suse.de
- require something in tabletpc (bnc#437162)
* Sun Nov  9 2008 coolo@suse.de
- moving some packages out of minimal+x
* Sun Nov  9 2008 coolo@suse.de
- fix build
* Thu Nov  6 2008 coolo@suse.de
- several packages added for bugs
* Sun Nov  2 2008 coolo@suse.de
- split out non_oss pattern as we have a new setup now
* Sun Nov  2 2008 coolo@suse.de
- sort out pam (again)
- adding Policykit-kde to kde3 desktop (bnc#440829)
* Sun Nov  2 2008 coolo@suse.de
- install liberation-fonts
* Tue Oct 28 2008 coolo@suse.de
- fixed long standing bug 388570 - make kerneloops default
* Tue Oct 28 2008 coolo@suse.de
- move java-plugin out of non-oss
- install the pullin packages with non-oss
- make packages required in non-oss-java
* Tue Oct 28 2008 coolo@suse.de
- hopefully sort out the pam-xx packages (only for DVD)
* Tue Oct 28 2008 coolo@suse.de
- remove yast2-wagon
* Mon Oct 27 2008 coolo@suse.de
- install smolt by default for the desktops
* Thu Oct 16 2008 cthiel@suse.de
- back to NetworkManager-kde for KDE4-BASIS
* Thu Oct 16 2008 cthiel@suse.de
- drop gstreamer-0_10-pulse from patterns (bcn#435943)
* Wed Oct 15 2008 cthiel@suse.de
- cleanup for Beta3
- switch from kpowesave to powerdevil on KDE4 (bnc#434753)
- added yast2-wagon to YAST_BASIS
- suggest sblim-sfcb and cim-schema in YAST_BASIS
- created TablePC pattern
- make sure to not have required packages in patterns, that are available
  online-only: DEVEL-CELL, DEVEL-Rails, DEVEL-Ruby, DEVEL-WEB, DEVEL-YaST,
  LAMP and VOIP (bnc#431865, bnc#432508, bnc#427691)
- added createrepo to REST-DVD (bnc#432164)
- added accerciser to DEVEL-GNOME
* Thu Oct  9 2008 cthiel@suse.de
- added sbl and espeak to REST-CD to make LiveCD more accessible
  (bnc#391327)
- ttf-arphic-uming to REST-DVD (bnc#431280)
* Thu Oct  9 2008 cthiel@suse.de
- moving lsb from suggested to recommended in X11-OPT (bnc#423075)
* Thu Oct  9 2008 cthiel@suse.de
- switch to NetworkManager-kde4 in KDE4-BASIS
- cleanup XGL/GNOME-XGL patterns
- added rsync and smartmontools to LiveCD (bnc#419201)
* Mon Oct  6 2008 cthiel@suse.de
- add some cyrus-sasl-* plugins to ENHANCED-BASIS (bnc#430895)
* Mon Oct  6 2008 cthiel@suse.de
- improved KDE patterns (bnc#432365)
- remove unwanted KDE4 apps from "KDE3 Desktop Environment" (bnc#432368)
* Tue Sep 30 2008 cthiel@suse.de
- pre Beta2 cleanup
- added split kernel
- added PolicyKit and polkit-default-privs to BASIS
* Mon Sep 29 2008 cthiel@suse.de
- KDE pattern update from Stephan Binner (bnc#430500)
* Fri Sep 26 2008 cthiel@suse.de
- added xorg-x11-driver-video-radeonhd
* Fri Sep 26 2008 cthiel@suse.de
- added polkit-default-privs to GNOME-BASIS and KDE4-BASIS (bnc#430161)
* Fri Sep 26 2008 cthiel@suse.de
- added atl2-kmp-* to REST-DVD (bnc#420039)
* Fri Sep 26 2008 cthiel@suse.de
- added skanlite to KDE4-IMAGE-OPT (bnc#422555)
* Fri Sep 26 2008 cthiel@suse.de
- added xorg-x11-xauth to X11
* Fri Sep 26 2008 cthiel@suse.de
- added iwl5000-ucode (bnc#426500)
- make sure to use yast2-qt for all installations
* Fri Sep 26 2008 cthiel@suse.de
- replace gnome-cups-manager with system-config-printer in GNOME-BASIS
* Tue Sep 23 2008 cthiel@suse.de
- rpcbind moved from ENHANCED-BASIS to BASIS, to enable minimal NFS
  installation
- pulseaudio added to KDE4-BASIS and KDE3-BASIS
- removed openSUSE-release-dvd from BASIS, only openSUSE-release has to be
  in the patterns
* Thu Sep 18 2008 cthiel@suse.de
- fix typo in BASIS
* Thu Sep 18 2008 cthiel@suse.de
- openSUSE-release got renamed again.
* Thu Sep 18 2008 cthiel@suse.de
- switch to openSUSE-build-key
* Wed Sep 17 2008 cthiel@suse.de
- FATE #304494: System Instrumentation (CIM Enablement), adding
  smis-providers to REST-DVD
* Wed Sep 17 2008 aj@suse.de
- Remove dropped package fast-user-switch-applet.
* Tue Sep 16 2008 cthiel@suse.de
- adding scout and command-not-found to ENHANCED-BASIS (bnc#424707)
- adding XFCE-Office pattern, to always install OpenOffice-gnome in case
  XFCE and OFFICE are selected (bnc#254454)
* Tue Sep 16 2008 cthiel@suse.de
- added simple-ccsm-kde to KDE4-XGL and KDE3-XGL (bnc#426316)
- replace kdebluetooth with kdebluetooth4 in KDE4-Utilities (bnc#422601)
- replace powersave with pm-profiler in ENHANCED-BASIS (bnc#417998)
- move to kde4-amarok on KDE4-Multimedia (bnc#421870)
* Fri Sep 12 2008 cthiel@suse.de
- remove dropped packages: lineak_kde, perl-Spreadsheet-WriteExcel,
  perl-XML-XQL, perl-XML-Generator, python-curl, gspcav-kmp-default,
  gspcav-kmp-pae, resapplet
- made various virtual provides implicit
- take care of package renames
- added man-pages-posix to enhanced_base
- replace portmap with rpcbind
- remove gail-devel, provided by gtk2-devel from DEVEL-GNOME
- remove dropped kyum from KDE3-DESKTOP
* Wed Sep 10 2008 aj@suse.de
- Remove dropped packages resmgr, hal-resmgr and yast2-support.
* Mon Sep  1 2008 aj@suse.de
- Remove agfa-fonts.
* Wed Aug 20 2008 cthiel@suse.de
- add back technical_writing [bnc#418571]
- add desktop-data-openSUSE-extra-gnome to GNOME-BASIS-OPT [bnc#394406]
* Mon Aug  4 2008 cthiel@suse.de
- remove dropped package gail-doc from DEVEL-GNOME
- remove unknown package itk-devel from DEVEL-Tcl
- remove unknown packages java-1_6_0-openjdk-alsa and
  java-1_6_0-openjdk-jdbc from NON-OSS-JAVA
- remove dropped package yast2-storage-evms from YAST_BASIS
- remove unknown package xfce4-dict from XFCE
- remove dropped package php5-ncurses from DEVEL-Web
- remove dropped packages uvcvideo-KMP-* from REST-DVD
- remove packages myldapklient, klipsi and kpsion from KDE3-DESKTOP
- remove dropped package kde4-kmilo from KDE4-LAPTOP
- remove packages kde4-kmobiletools and kde4-korn from KDE4-BASIS
- remove packages kde4-kworldclock and kde4-kpilot from KDE4-Utilities
* Mon Aug  4 2008 cthiel@suse.de
- XFCE pattern fixes (bnc#404447)
- take out the games from the kde x86_64 live
- moving some koffice and NON-OSS stuff to the online repo
- moving some TeX packages to the online repo
- removing duplicate gconf2-branding-openSUSE from GNOME-BASIS (bnc#412442)
- removing xgl from XGL pattern
- switch from sun to openjdk
- removing quite some patterns from the DVD for now
- adding laptop-mode-tools to LAPTOP (fate#303035)
* Fri Aug  1 2008 coolo@suse.de
- install biosdevname by default
* Mon Jun 30 2008 aj@suse.de
- Fix last change to c/c++ devel pattern.
* Mon Jun 30 2008 aj@suse.de
- Add xfce packages (bnc#404447)
* Mon Jun 30 2008 aj@suse.de
- Add dejagnu and expect to c/c++ devel pattern (bnc#403368).
* Thu Jun 26 2008 coolo@suse.de
- share mechanism with sled
- adding biosdevname
* Mon Jun 23 2008 coolo@suse.de
- drop unsupport pattern packages
* Tue Jun 10 2008 coolo@suse.de
- banshee-1 plugins are free of the mono-devel dep again
* Sun Jun  8 2008 coolo@suse.de
- requiring acroread on ppc is not a good idea, reverting
  the last change for non-oss pattern
* Thu Jun  5 2008 coolo@suse.de
- always require something in patterns, otherwise they get
  autoselected and this is suprising (bnc#392989)
* Wed Jun  4 2008 coolo@suse.de
- put alsa firmware on live cd (bnc#396109)
* Wed Jun  4 2008 coolo@suse.de
- make promo dvd an own pattern package
* Wed Jun  4 2008 coolo@suse.de
- don't require thinkreader support
* Tue Jun  3 2008 coolo@suse.de
- do not put update-test on DVD
* Wed May 28 2008 coolo@suse.de
- dropped kst for wallpapers
* Tue May 27 2008 coolo@suse.de
- replace #ifdef with #if defined (bnc#394623)
* Mon May 26 2008 coolo@suse.de
- use gdm for xfce (bnc#393956)
* Mon May 26 2008 coolo@suse.de
- scsi was split
* Thu May 22 2008 aj@suse.de
- Remove perl-MP3-Info from perl-devel since it's non-existing.
* Wed May 21 2008 coolo@suse.de
- adding icons again
* Sun May 18 2008 coolo@suse.de
- rename the internal name of kde cd to kde4
* Tue May 13 2008 coolo@suse.de
- final round of bug fixes for beta3
* Sat May 10 2008 coolo@suse.de
- adding "KDE3 User Guide" to the KDE3 pattern (bnc#388663)
* Sat May 10 2008 coolo@suse.de
- fixing typos
* Thu May  8 2008 coolo@suse.de
- call lint tool to avoid broken factory patterns
* Tue May  6 2008 coolo@suse.de
- fixing typo and adding yast dependencies
* Sun May  4 2008 coolo@suse.de
- use xchat under GNOME
* Sun May  4 2008 coolo@suse.de
- Install banshee-1 by default in GNOME (bnc#384845)
* Sun Apr 27 2008 coolo@suse.de
- fix bug putting yast2-qt on GNOME (bnc#381688)
* Fri Apr 25 2008 coolo@suse.de
- removing gnome2-SuSE, adding kernel-syms,
  renamed control-center2-devel
* Wed Apr 23 2008 coolo@suse.de
- updated icons
* Sat Apr 19 2008 coolo@suse.de
- several bug fixes
* Thu Apr 17 2008 coolo@suse.de
- downgrading apparmor to allow install
* Wed Apr 16 2008 coolo@suse.de
- redoing ordering of patterns
* Tue Apr 15 2008 coolo@suse.de
- removing dropped packages
* Mon Apr 14 2008 coolo@suse.de
- adding Policykit-kde and gnome-packagekit
* Sat Apr 12 2008 coolo@suse.de
- icon changes by Duncan
* Sun Apr  6 2008 coolo@suse.de
- sorting out dropped/renamed packages
- take kde4-opensuse-updater
* Tue Apr  1 2008 coolo@suse.de
- sorting out dropped/renamed packages
* Tue Apr  1 2008 coolo@suse.de
- moving x11 dependencies out of enhanced_base
* Fri Mar 28 2008 coolo@suse.de
- feedback from the KDE team
* Thu Mar 27 2008 coolo@suse.de
- adding tons of packages after feedback on opensuse-factory
* Thu Mar 20 2008 coolo@suse.de
- readding compiz to kde cd
* Thu Mar 20 2008 coolo@suse.de
- rework package, so I can build patterns out of SVN
* Thu Mar 20 2008 coolo@suse.de
- adding PolicyKit-gnome to the GNMOE desktop (bnc#372514)
* Wed Mar 19 2008 coolo@suse.de
- include libcompizconfig in pattern that installs compiz
  (bnc #371550)
* Tue Mar 18 2008 coolo@suse.de
- remove dropped packages, flag i586 only packages
* Tue Mar 18 2008 coolo@suse.de
- wesnoth(-data) is too big for the DVD ;(
* Sun Mar 16 2008 coolo@suse.de
- I almost forgot: non-oss is back on DVD
* Sun Mar 16 2008 coolo@suse.de
- renamed package
* Sat Mar 15 2008 coolo@suse.de
- dropped package removed
* Fri Mar 14 2008 coolo@suse.de
- take gnome branding openSUSE in the patterns
* Thu Mar 13 2008 coolo@suse.de
- another case of permissions
* Wed Mar 12 2008 coolo@suse.de
- check arch specific CD setup
* Tue Mar 11 2008 coolo@suse.de
- updating patterns
* Mon Mar 10 2008 coolo@suse.de
- actually doing svn up before submitpac
* Fri Mar  7 2008 coolo@suse.de
- as a direct consequence of the discussion on the pattern
  concept between software management group: reduce heavily the
  use of requirements to packages
* Thu Mar  6 2008 coolo@suse.de
- only take gnome games on i386
* Sun Mar  2 2008 coolo@suse.de
- do not rely on file permissions
* Fri Feb 29 2008 coolo@suse.de
- remove xgl from kde cd till we have compiz-kde4 in factory
- adding games to gnome and kde cd
* Thu Feb 28 2008 coolo@suse.de
- adding xorg driver for virtualbox speed (#365562)
* Tue Feb 19 2008 coolo@suse.de
- add yast2 requires
* Sun Feb 17 2008 coolo@suse.de
- virtualbox renamed to virtualbox-ose
* Sun Feb 17 2008 coolo@suse.de
- xen_server should bring its kernel (bnc#362028)
* Sun Feb 17 2008 coolo@suse.de
- move more of the unknown packages to suggests
- include not otherwise excluded suggests
* Fri Feb 15 2008 coolo@suse.de
- remove several dropped packages
* Wed Feb  6 2008 coolo@suse.de
- adding yast2-qt to the GNOME CD again (gtk horribly broken)
- removing xsane/gimp from the GNOME CD
* Tue Feb  5 2008 coolo@suse.de
- switch to kde4 for the CD
* Tue Feb  5 2008 coolo@suse.de
- several updates from the KDE team to the KDE4 patterns
* Tue Feb  5 2008 coolo@suse.de
- libzypp-devel is a good candidate for Devel-yast
* Mon Feb  4 2008 coolo@suse.de
- Rodrigo wants pulseaudio in a default GNOME install
* Sun Feb  3 2008 coolo@suse.de
- kde_ patterns should be obsolete
* Sun Feb  3 2008 coolo@suse.de
- make sure kde subpatterns freshen the right version
* Sun Feb  3 2008 coolo@suse.de
- don't include non-oss on the DVD (#357987)
* Fri Feb  1 2008 coolo@suse.de
- don't be too eager in 32bit packages
* Thu Jan 31 2008 coolo@suse.de
- remove more non-existant packages
* Thu Jan 31 2008 coolo@suse.de
- removed dropped packages
- replace renamed packages
- move some more packages to suggests
* Wed Jan 30 2008 coolo@suse.de
- warn about missing packages
* Tue Jan 29 2008 coolo@suse.de
- copy kde patterns into kde3 and kde4
* Sun Jan 27 2008 coolo@suse.de
- also include the KDE yast on the CD
* Fri Jan 25 2008 coolo@suse.de
- move gimp off the CDs
- default to sun java 1.6
- move yast2-qt off the GNOME CD (for installed system)
* Mon Jan 21 2008 coolo@suse.de
- some fixes to get closer to automated kiwi builds
* Mon Jan 21 2008 coolo@suse.de
- list also recommends as required
* Sat Jan 19 2008 coolo@suse.de
- reenable packages that were broken for alpha1
- remove dvd5, dvd9 and cd - they weren't used on 10.3
- let the pattern package require the packages required
  and recommended to make kiwi mbuilds easier
* Tue Jan 15 2008 coolo@suse.de
- several fixes and hot fixes to get alpha1 installable
* Thu Dec 20 2007 coolo@suse.de
- yast2-bluetooth is not provided by anything
* Wed Dec 19 2007 coolo@suse.de
- xml -> technical_writing
* Wed Dec 19 2007 coolo@suse.de
- some fixes for dropped packages
* Wed Dec  5 2007 coolo@suse.de
- include 32bit pam modules
* Wed Dec  5 2007 coolo@suse.de
- slim down default install of both the CDs and the DVD
* Tue Dec  4 2007 coolo@suse.de
- remove some packages without use in the hope to find out if they
  are still in use
* Tue Nov 27 2007 coolo@suse.de
- adding libicu to the pattern, so that zypper will also work
* Mon Nov 26 2007 coolo@suse.de
- /lib/cpp breaks our patterns
* Wed Nov 21 2007 coolo@suse.de
- random cleanup in dropped and split packages
* Mon Nov 12 2007 coolo@suse.de
- rename xml to technical_writing (#332935)
* Fri Nov  9 2007 coolo@suse.de
- adding subversion for yasties (#340575)
* Wed Nov  7 2007 coolo@suse.de
- also add bluez-gnome to gnome default (#226137)
* Thu Oct 11 2007 coolo@suse.de
- install gnome-keyring-pam by default (#332596)
- don't let sw_management_kde and sw_management_gnome conflict (#279644)
- add bluez-gnome to gnome_laptop (#299100)
- remove gtk-engines and gdk-pixbuf from default (#330731)
- add gsynaptics to gnome laptop (#330757)
- not installing autoyast by default (#308684)
- degrade yast2-theme-openSUSE to a recommend (#331549)
- adding 3ddiag to default X11 (#331559)
- adding resapplet to gnome laptop (#328455)
- degrade gimp to a recommend (#331540)
- misc cleanup
* Wed Sep 26 2007 cthiel@suse.de
- fix missing icons (#216557)
* Tue Sep 25 2007 coolo@suse.de
- readding some DEVEL-Perl suggests to the DVD (#327939)
* Tue Sep 25 2007 coolo@suse.de
- move cabextract to the CD set (#327524)
* Mon Sep 24 2007 coolo@suse.de
- really remove update_test pattern
* Mon Sep 24 2007 cthiel@suse.de
- really remove update_test
* Mon Sep 24 2007 cthiel@suse.de
- added bcm43xx-fwcutter to REST-CD (#327506)
* Mon Sep 24 2007 coolo@suse.de
- do not show arcad_eval on x86_64 (#248873)
* Thu Sep 20 2007 coolo@suse.de
- suggest libsmbios-bin on laptops
* Thu Sep 20 2007 coolo@suse.de
- improve KDE Devel pattern after feedback from dirk
* Thu Sep 20 2007 coolo@suse.de
- remove translations from DVD patterns for english only installations
  (#326425)
* Thu Sep 20 2007 coolo@suse.de
- exclude the update_test pattern
* Wed Sep 19 2007 cthiel@suse.de
- only require libspe on power in DEVEL-Cell
* Wed Sep 19 2007 cthiel@suse.de
- fixing problems reported by coolo
* Wed Sep 19 2007 coolo@suse.de
- adding deltarpm to the CDs
* Tue Sep 18 2007 cthiel@suse.de
- added squashfs to REST-DVD, to support kiwi image generation
* Tue Sep 18 2007 coolo@suse.de
- don't install classpath-webplugin (#325912)
* Fri Sep 14 2007 cthiel@suse.de
- added antivir-gui to NON-OSS as requested PM
* Fri Sep 14 2007 cthiel@suse.de
- added missing icons to patterns (#216557)
* Fri Sep 14 2007 cthiel@suse.de
- added more aufs to REST-DVD
- added opera to REST-DVD
* Thu Sep 13 2007 cthiel@suse.de
- added irqbalance to ENHANCED-BASIS
* Thu Sep 13 2007 cthiel@suse.de
- added aufs and aufs-kmps to REST-DVD
- recommended pattern devel-cell in rest_dvd
* Wed Sep 12 2007 cthiel@suse.de
- move numactl from ENHANCED-BASIS-OPT to ENHANCED-BASIS
* Wed Sep 12 2007 cthiel@suse.de
- readded wesnoth to REST-DVD
* Tue Sep 11 2007 cthiel@suse.de
- adding ipw3945d, -kmps and ipw-firmware to REST-CD (#307429)
* Tue Sep 11 2007 coolo@suse.de
- adding fluendo mp3 to NON-OSS pattern
* Mon Sep 10 2007 cthiel@suse.de
- added intlclock to GNOME-BASIS (#304676)
* Wed Sep  5 2007 cthiel@suse.de
- added lukemftp to ENHANCED-BASIS (#307925)
* Tue Sep  4 2007 cthiel@suse.de
- removed gnomebaker from GNOME-BASIS and added brasero instead (#302492)
- removed gnome_games from rest_cd_gnome/requires
* Mon Sep  3 2007 cthiel@suse.de
- added kdebase4-workspace to KDE4-BASIS (#307054)
* Mon Sep  3 2007 cthiel@suse.de
- added mouseemu to ENHANCED-BASIS on power (#303737)
* Mon Sep  3 2007 cthiel@suse.de
- moved resmgr from ENHANCED-BASIS to BASIS (#307095)
* Mon Sep  3 2007 cthiel@suse.de
- added mdadm to ENHANCED-BASIS (#306749)
* Thu Aug 30 2007 cthiel@suse.de
- added virtualbox + virtualbox-kmp to REST-DVD (#306071)
* Tue Aug 28 2007 cthiel@suse.de
- added libgail-gnome to GNOME-BASIS (#302205)
- move yast2-bootloader from YAST2-BASIS to YAST_INSTALL_WF (#292435)
* Mon Aug 27 2007 aj@suse.de
- Add more packages to yast2 devel pattern (#304645).
- suspend is i386/x86-64 only (#304756).
* Mon Aug 27 2007 cthiel@suse.de
- install resmgr by default (#304695)
- install psmisc by default (#304694)
- install gconf-editor by default (#304663)
- readd gnome-games to GNOME CD (#304662)
- install man by default (#304687)
- added yast2-fingerprint-reader to YAST_BASIS (#304686)
- sw_management pattern recommended in base and enhanced_base pattern (#304307)
- install audit by default (#304207)
- kpartx and dmraid to ENHANCED-BASIS (#303857)
- seamonkey* packages to REST-DVD (#304221)
- moved x11-tools from X11-OPT to X11 (#302566)
- added p54-kmp-default to REST-CD (#300798)
- added atmel-firmware to REST-CD (#298726)
- added lessons4lizards_en to REST-DVD (#302465)
- added alsa-plugins to ENHANCED-BASIS (#302569)
- moved memtest86+ from REST-DVD to REST-CD (#304219)
* Thu Aug 23 2007 cthiel@suse.de
- added adaptec-firmware, qlogic-firmware and mpt-firmware to REST-CD
  (#298726)
* Thu Aug 23 2007 cthiel@suse.de
- removed squashfs, since it's moving into the kernel packages
* Wed Aug 22 2007 cthiel@suse.de
- added hylafax, hylafax-client and capi4hylafax to REST-DVD
* Wed Aug 22 2007 cthiel@suse.de
- added iwlwifi-kmp-* to REST-DVD
* Wed Aug 22 2007 cthiel@suse.de
- removed xfce4-panel-plugins from XFCE and readded xfce4-panel-plugin-*
* Wed Aug 22 2007 cthiel@suse.de
- added strace to ENHANCED-BASIS
* Tue Aug 21 2007 cthiel@suse.de
- removed dropped initial from rest-dvd
- moved kio_beagle, beagle-firefox, kdebase3-beagle and kerry to KDE-BASIS
* Tue Aug 21 2007 cthiel@suse.de
- moved lvm2 to REST-CD
* Tue Aug 21 2007 cthiel@suse.de
- added bundle-lang-common-en to REST-CD
* Tue Aug 21 2007 cthiel@suse.de
- moved lvm2 to ENHANCED-BASIS (#301382)
- removed nautilus-cd-burner and added gnomebaker to GNOME-BASIS pattern
  (#269051)
- added ggreeter to GNOME-BASIS (#302026)
- addded gnome-commander to GNOME-BASIS (#301664)
- added freeglut to X11 (#301647)
* Fri Aug 17 2007 cthiel@suse.de
- added opensuse-codecs-installer to KDE & GNOME Multimedia
* Fri Aug 17 2007 cthiel@suse.de
- added yast2-add-on-creator to rest_dvd (#301047)
* Thu Aug 16 2007 cthiel@suse.de
- added yast2-python-bindings to rest_dvd (#301029)
* Thu Aug 16 2007 cthiel@suse.de
- added squashfs + kmp to rest_cd (#300942)
* Thu Aug 16 2007 cthiel@suse.de
- adding kexec-tools, kdump-helpers and yast2-kdump to rest_dvd
* Wed Aug 15 2007 coolo@suse.de
- adding kinternet back to the medium
* Wed Aug 15 2007 coolo@suse.de
- tomcat5 -> tomcat55 in devel_java
- make dvd5 and dvd9 alike
* Wed Aug 15 2007 coolo@suse.de
- don't install manuals by patterns, but by provides on locales
  (#296873)
* Wed Aug 15 2007 coolo@suse.de
- don't install kmilo by default (#298508)
* Tue Aug 14 2007 coolo@suse.de
- move kdegraphics3-scan from CD to DVD only
- adding cups from DVD only to CD
* Tue Aug 14 2007 coolo@suse.de
- adding some packages that are installed by hardware dependencies
* Tue Aug 14 2007 coolo@suse.de
- hostap is dropped
* Mon Aug 13 2007 coolo@suse.de
- include ntfs-3g by default
* Sun Aug 12 2007 coolo@suse.de
- remove dropped packages
* Sat Aug 11 2007 coolo@suse.de
- fix file list
* Fri Aug 10 2007 coolo@suse.de
- move cups-autoconf into DVD enhanced_basis
* Fri Aug 10 2007 coolo@suse.de
- split rest_cd into kde and gnome to simplify configs
* Wed Aug  8 2007 coolo@suse.de
- adding a rest_dvd pattern for dvd5
- dropping yast2-power-management
* Tue Aug  7 2007 coolo@suse.de
- voip-server is no more - asterix is dropped
* Mon Aug  6 2007 coolo@suse.de
- replace kdenetwork3-vnc with kde4-krfb/kde4-krdc (#297684)
* Mon Aug  6 2007 coolo@suse.de
- move latex2html-pngicons, sensors off the GNOME CD
- move ed to be recommended on DVD - let's try what or who needs ed
- remove all games from the GNOME CD
* Sun Aug  5 2007 coolo@suse.de
- move smpppd off the CD
- extend the base patterns
- rename curl-devel and libpurple-devel
- readd nautilus-open-terminal to GNOME
- move exiftool off the CD
- readd numlockx to the CD
- track DVD packages from bug reports
* Wed Aug  1 2007 coolo@suse.de
- hide helix-banshee to avoid a conflict
* Wed Aug  1 2007 coolo@suse.de
- move nautilus-open-terminal and alacarte off the CD
* Tue Jul 31 2007 coolo@suse.de
- shuffle around more GNOME bits
* Tue Jul 31 2007 coolo@suse.de
- split out gtk1 stuff
* Tue Jul 31 2007 coolo@suse.de
- need to split out more GNOME patterns to fit on CD (e.g. inkscape, dia)
* Mon Jul 30 2007 coolo@suse.de
- split GNOME multimedia for OSS medium and replace gaim with pidgin
* Mon Jul 30 2007 coolo@suse.de
- yast2-registration should be on the media in any case
* Mon Jul 30 2007 coolo@suse.de
- adding a rest_cd pattern to track invisible dependencies
  needed on the CD
* Mon Jul 30 2007 coolo@suse.de
- also add recommends to the opt patterns
* Mon Jul 30 2007 coolo@suse.de
- move packages on CD back out of opt
* Fri Jul 27 2007 coolo@suse.de
- split several patterns into the required packages and
  optionals not on CD (now)
* Fri Jul 27 2007 coolo@suse.de
- include apparmor on CDs
* Fri Jul 27 2007 coolo@suse.de
- Add fonts_opt pattern
- Remove obvious dependencies from enhanced_base
- Document some decisions
* Thu Jul 26 2007 aj@suse.de
- Make files in GAMES pattern recommended.
- Add enhanced_base_opt pattern.
- Add GNOME-cd and KDE-cd packages.
* Fri Jul 20 2007 aj@suse.de
- Use amarok-yaup instead of amarok-xine (#293531).
* Tue Jul 17 2007 aj@suse.de
- Add libgcc42 to basis.
* Tue Jul 10 2007 aj@suse.de
- Adjust KDE4 patterns as suggested by Stephan Binner.
* Tue Jul 10 2007 aj@suse.de
- Add atftp to File Server (#288154).
* Thu Jul  5 2007 aj@suse.de
- Remove dmraid, multipath-tools from default install.
* Wed Jul  4 2007 aj@suse.de
- Remove limal-bootloater.
* Mon Jul  2 2007 aj@suse.de
- Remove freetype from fonts pattern (#283827).
- Add perl-TermReadLine-Gnu to Enhance-Basis (#285827).
* Mon Jul  2 2007 aj@suse.de
- Move some packages from required to recommended as suggested by
  Lars Vogdt for the 1 CD installs.
* Sat Jun 30 2007 aj@suse.de
- Remove lvm2, mdadm from install, YaST will install if needed.
- Add office pattern, add splitted OpenOffice_org packages.
* Fri Jun 29 2007 aj@suse.de
- Add xournal to GNOME-LAPTOP (#288160).
* Thu Jun 28 2007 aj@suse.de
- Add vm-install and virt-manager to XEN pattern (#286987).
- Require xkeyboard-config in X11 (#285001).
- Require hal-resmgr in enhanced basis (#285057), add setserial
  as recommended.
- Make some packages only recommended in GNOME Imaging (#285251).
* Thu Jun 14 2007 aj@suse.de
- Add KDE4 Basis pattern (#283896).
- Cleanup devel-qt4 pattern (#283889).
* Mon Jun 11 2007 aj@suse.de
- nfs-utils -> nfs-client for basis.
- nfs-utils -> nfs-kernel-server for NFS Server pattern.
* Sat Jun  9 2007 aj@suse.de
- Add Cell Development pattern (#273697).
- Remove 915resolution from X11 pattern (#266362).
* Fri Jun  8 2007 aj@suse.de
- Make X11 pattern smaller (#266362).
- Add linux-atm-lib to ENHANCED_BASE (#225781).
- Add banshee-plugins-DAAP for GNOME (#273896).
- Add metacity-devel andlibgnomekbd-devel for DEVEL-GNOME (#278972,
  [#278973]).
- Add xfce4-volstatus to XFCE (#279080).
* Wed May 30 2007 lrupp@suse.de
- remove dropped package kdepim3-networkstatus
* Mon May 28 2007 aj@suse.de
- Add yast2-mcs-manager to XFCE pattern.
* Tue May 15 2007 aj@suse.de
- Remove evms packages from default installs.
* Mon May 14 2007 lrupp@suse.de
- move deleted package yast2-mail-aliases to Prc
* Fri May 11 2007 aj@suse.de
- Add yast2-metapackage-handler to YaST2 pattern.
* Thu May 10 2007 aj@suse.de
- remove evms-gui from X11 pattern (#266362).
- remove mozilla-xulrunner181 from X11 pattern (#266362).
- remove libqt4 from X11 pattern, it should come in via
  dependencies (#266362).
* Thu May 10 2007 aj@suse.de
- Make freetype only recommended (#273170).
* Wed May  2 2007 aj@suse.de
- Add cracklib-dict-large as recommended.
* Wed May  2 2007 aj@suse.de
- Readd initviocons, it's needed elsewhere.
* Wed May  2 2007 aj@suse.de
- Add kde4 games.
- Remove yast2-powertweak.
* Tue May  1 2007 aj@suse.de
- Make jfs and xfs as suggested only.
* Sat Apr 28 2007 aj@suse.de
- Some fixes for bug 269774:
  * Install initviocons on powerpc only.
  * Remove tpctl from laptop, hwinfo should add the require.
* Tue Apr 24 2007 aj@suse.de
- alsa-32bit -> libasound2-32bit.
* Thu Apr 19 2007 aj@suse.de
- Rework opensuse-updater patterns.
* Tue Apr 17 2007 aj@suse.de
- Add finger to default install (#224873).
* Mon Apr 16 2007 aj@suse.de
- Add licenses to BASIS.
* Mon Apr 16 2007 aj@suse.de
- Add mrtg to Network Admin pattern (#264670).
- Obsolete/provide zmd patterns.
* Fri Apr 13 2007 aj@suse.de
- Disable zmd patterns.
* Wed Apr 11 2007 aj@suse.de
- Add dasher, seahorse for GNOME (#262024).
- Add yast2-ftp-server.
- Remove wireless packages that yast should install as needed.
* Tue Apr 10 2007 aj@suse.de
- Add nspluginwrapper for Firefox.
* Thu Apr  5 2007 aj@suse.de
- Add nss-mdns-32bit for Firefox (#249686).
* Fri Mar 30 2007 aj@suse.de
- Add xfce-mcs-plugins to XFCE pattern (#258845).
* Thu Mar 29 2007 aj@suse.de
- Remove packages from RPM development environment that are not
  installed anymore.
* Mon Mar 26 2007 aj@suse.de
- Add filters to default install (#257038).
* Thu Mar 22 2007 aj@suse.de
- Add opensuse-updater-gnome.
* Fri Mar 16 2007 aj@suse.de
- Add yast2-control-center binary modules to desktops (#254137).
* Fri Mar 16 2007 aj@suse.de
- Install multimedia for KDE (#253022).
* Wed Feb 21 2007 aj@suse.de
- Do not install zmd by default.
* Fri Feb 16 2007 aj@suse.de
- Add gnome_yast (#245411, #245988).
* Fri Feb 16 2007 aj@suse.de
- Fix some GNOME and KDE patterns.
* Thu Feb 15 2007 aj@suse.de
- Remove yast2-theme* from minimal install.
- pkgconfig->pkg-config.  Remove duplicates.
* Wed Feb 14 2007 aj@suse.de
- Really install OOo with GNOME (#245408).
- Install banshee with GNOME (#245407).
* Wed Feb 14 2007 aj@suse.de
- Add mtools to CONSOLE (#244591).
* Mon Feb 12 2007 aj@suse.de
- Move create_patterns.sh to patterns package.
* Mon Feb 12 2007 aj@suse.de
- Remove gtk-sharp packages (#244537).
- Remove hal-gnome from gnome_basis (#244417).
* Fri Feb  9 2007 aj@suse.de
- Fix build problem.
* Fri Feb  9 2007 aj@suse.de
- Build patterns package separately.
- Do not build as root.
* Thu Feb  8 2007 aj@suse.de
- Move ia32el to NON-OSS (#243590).
* Thu Feb  8 2007 aj@suse.de
- Localization update.
- Build as root, patterns_utf8 needs to access its mo files.
* Tue Feb  6 2007 aj@suse.de
- Enhance XFCE pattern (#224558).
* Thu Feb  1 2007 aj@suse.de
- Add ncftp to CONSOLE (#224692).
- Add grepmail, links to CONSOLE (#239564).
- Add black-box, WindowMaker to X11 (#212529).
* Wed Jan 31 2007 aj@suse.de
- Version 10.3.
* Fri Jan 26 2007 aj@suse.de
- Remove redundancies.
* Thu Jan 25 2007 aj@suse.de
- Split GNOME into smaller patterns.
* Thu Jan 25 2007 aj@suse.de
- Follow package renames.
* Thu Jan 25 2007 aj@suse.de
- Add separate multimedia pattern (by Lars Vogdt).
* Tue Jan 23 2007 aj@suse.de
- Further minimization of BASIS pattern.
* Fri Jan 19 2007 aj@suse.de
- Use weak dependencies for non-oss pattern.
- Enhance patterns.
* Thu Jan 18 2007 aj@suse.de
- Split base system into minimal base and enhanced base.
- Start providing a YaST pattern for installation workflow.
* Thu Jan 18 2007 aj@suse.de
- Add some python GNOME packages (#235922).
* Wed Jan 17 2007 aj@suse.de
- Use smaller KDE patterns.
- Add compiz-kde, enable KDE XGL pattern.
* Wed Jan 17 2007 aj@suse.de
- Add separate update test pattern (#220798).
* Wed Jan 17 2007 aj@suse.de
- Add vino to GNOME Desktop pattern (#225382).
- Make 32-bit pattern visible (#224863).
- Remove build from kernel pattern, it's in the base devel pattern
  (#227045).
- Do not install ash by default (#222905).
- Fix typo in patterns (#224373).
* Fri Jan 12 2007 aj@suse.de
- Update localization.
- Add SLED pattern names.
* Wed Nov 29 2006 aj@suse.de
- Add xsane for GNOME desktop, sane-frontends for X11 (#224411).
- Fix some pattern dependencies, especially to not install gnome base
  system if XGL is selected under KDE (#224412).
* Tue Nov 28 2006 aj@suse.de
- Install opensuse sw_management tools as well.
- i18n update (km).
- Install xgl pattern for KDE as well.
* Mon Nov 27 2006 aj@suse.de
- i18n update.
* Sun Nov 26 2006 aj@suse.de
- Remove pdfs of manuals.
* Sun Nov 26 2006 aj@suse.de
- Add manuals (#223187).
* Fri Nov 24 2006 aj@suse.de
- Require java-1_4_2-gcj-compat-32bit for OOo (#223440).
- Fix mail server pattern syntax (#223497).
- Fix typo in kde pattern (#220277).
- Add wesnoth to games pattern (#223573).
- Fixes for devel patterns, especially devel-kde (#223594).
* Thu Nov 23 2006 aj@suse.de
- Fix xfce pattern requires (#223438).
- Install pmtools only on supported archs (#223186).
- Add libzypp-zmd-backend to zmd pattern (#223323).
- Translation update.
- Add mozilla-xulrunner181 to X11 pattern (#223437).
* Tue Nov 21 2006 aj@suse.de
- Install fast-user-switch-applet by default on GNOME (#222878).
* Mon Nov 20 2006 aj@suse.de
- Translation update.
* Fri Nov 17 2006 aj@suse.de
- Translation update.
* Fri Nov 17 2006 aj@suse.de
- Fix bugs in patterns (#221824).
- Remove dropped libopensync-plugin-synce package.
* Fri Nov 17 2006 aj@suse.de
- Add deskbar-applet for GNOME (#216067).
- Add pkgconfig to X11 pattern.
- Localization update.
* Wed Nov 15 2006 aj@suse.de
- Localization update.
- Only install asterix-zaptel on i386 and x86-64 (#221288).
- Add some more packages to mail server (#221247).
* Wed Nov 15 2006 aj@suse.de
- Add qtcurve-gtk2-32bit to KDE pattern (#221234).
- Fix XGL patterns.
- Add kdetoys3 to kde desktop pattern (#220892).
- Add yast2-storage-evms to yast2 basis (#220425).
* Tue Nov 14 2006 aj@suse.de
- Fix XGL pattern visibility (#220852).
- Change names of package management patterns (#220741).
- Install yast2-samba-server by default with yast-basis pattern
  (#212388).
* Tue Nov 14 2006 aj@suse.de
- Further rework of software management patterns.
* Tue Nov 14 2006 aj@suse.de
- Add liferea and pan to GNOME desktop pattern (#220275).
* Mon Nov 13 2006 aj@suse.de
- Add tcl devel pattern (#220427).
* Mon Nov 13 2006 aj@suse.de
- Remove/rename some java packages.
- Rework software management patterns.
* Mon Nov 13 2006 aj@suse.de
- Make gnome-media only recommended.
- Remove compiz-gnome from KDE pattern (#220291).
- Add VOIP server and client patterns (#220241).
- Add iftop to Network-Admin.
- Remove electricfence from devel-c.
- Add pin to sw management patterns (#220292).
- Add more packages to console pattern (#220292).
- Readd Desktop Effects pattern.
* Sun Nov 12 2006 aj@suse.de
- Remove gnome-doc-utils from gnome_basis (#220263).
- Move some gnome packages around in the GNOME patterns (#220259).
* Sun Nov 12 2006 aj@suse.de
- Localization update.
- Add gnome-mount to GNOME-BASIS (#215392).
- Recommend NON-OSS, NON-OSS-Java for KDE and GNOME desktops (#219711).
- Add nano to console tools (#215652).
- Enhance imaging patterns.
- Remove libopensync-plugin-kdepim from GNOME-DESKTOP pattern (#220202).
* Thu Nov  9 2006 aj@suse.de
- Move kdeedu3 to KDE-Desktop pattern (#219361).
- Add  manufacturer-PPDs to Base pattern.
* Wed Nov  8 2006 aj@suse.de
- Add RealPlayer to KDE patterns.
- Add some 32-bit patterns for RealPlayer to patterns (#219009).
* Wed Nov  8 2006 aj@suse.de
- Remove banshee-subpackages from patterns, the package itself has
  the recommends now.
- translation update.
* Tue Nov  7 2006 aj@suse.de
- Install java-plugin by default on i586 (#216499).
- Remove kernel-xen again (#217934).
* Tue Nov  7 2006 lrupp@suse.de
- fix missing ":" in DEVEL-Kernel
* Tue Nov  7 2006 aj@suse.de
- Add alsa-32bit for x86-64 (#218739).
* Tue Nov  7 2006 aj@suse.de
- Install compiz-gnome under KDE as well.
- Fix require of software management pattern.
* Tue Nov  7 2006 aj@suse.de
- Handle banshee/helix-banshee better (#206954).
* Tue Nov  7 2006 aj@suse.de
- Further package for kernel-devel (#217999).
* Tue Nov  7 2006 aj@suse.de
- Add packages to kernel-devel (#217999).
- Add yast2-sudo to yast2 basis pattern.
- Add pinentry-qt, pinentry-gtk2 to patterns (#217373).
- Recommend AppArmor instead of using supplements.
* Mon Nov  6 2006 cthiel@suse.de
- Add dockutils to LAPTOP
* Mon Nov  6 2006 aj@suse.de
- Translation update.
* Sat Nov  4 2006 aj@suse.de
- Translation updates.
* Thu Nov  2 2006 cthiel@suse.de
- Add totem-plugin to GNOME-DESKTOP and GNOME-Multimedia (#217457)
- Add cups-drivers and gutenprint to BASIS, gutenprint to PRINT_SERVER (#217321)
* Wed Nov  1 2006 aj@suse.de
- Fix description of misc server pattern (#216556).
* Tue Oct 31 2006 cthiel@suse.de
- Add nautilus-sendto to GNOME-BASE (#216106)
* Tue Oct 31 2006 cthiel@suse.de
- Add gnome-doc-utils-devel to DEVEL-GNOME
* Tue Oct 31 2006 cthiel@suse.de
- Add vte-devel to DEVEL-GNOME
- Add intltool to DEVEL-GNOME (#216592)
* Tue Oct 31 2006 cthiel@suse.de
- Add alacarte to GNOME-DESKTOP (#216065)
* Mon Oct 30 2006 aj@suse.de
- Add nfs-server to fileserver pattern (#215648).
* Mon Oct 30 2006 aj@suse.de
- Fix sw_management patterns.
- Add kernel-xen to XEN pattern (#208380)
- Fix some description.
* Mon Oct 30 2006 aj@suse.de
- Fix enhances of sw_management pattern.
* Mon Oct 30 2006 aj@suse.de
- Add software management patterns.
- Reorder patterns.
* Sat Oct 28 2006 aj@suse.de
- Remove kernel again.
- Add glibc-locale-32bit to BASE pattern (#215910).
* Fri Oct 27 2006 aj@suse.de
- Add kernel to BASIS (#215644).
* Fri Oct 27 2006 aj@suse.de
- Changes to GNOME patterns:
  * Install totem (#215431).
  * Install gstreamer plugins (#215427).
  * Install banshee-plugins-extra (#215426).
- Add hal-resmgr to base pattern (#215396).
- Add defines for s390.
- localisation update.
- Add kpowersave again to KDE pattern.
- Rework Network admin pattern and add to CDs (#215653).
* Thu Oct 26 2006 aj@suse.de
- Replace kdepim3-sync with kitchensync.
- Install sync packages by default for GNOME and KDE (#215252).
- Install compiz and friends by default.
* Wed Oct 25 2006 aj@suse.de
- i18n update.
- Remove some empty lines that confused the parser (Bug #214877).
* Tue Oct 24 2006 aj@suse.de
- One more typo...
- Disable kpowersave for now.
- localization update.
* Tue Oct 24 2006 aj@suse.de
- Add freshens for gnome_games.
* Tue Oct 24 2006 aj@suse.de
- Fix one more typo.
* Tue Oct 24 2006 lrupp@suse.de
- fix typo in remote-desktop pattern
- remove two dropped packages from lamp-pattern
* Tue Oct 24 2006 aj@suse.de
- Better group patterns.
* Tue Oct 24 2006 aj@suse.de
- Cleanup display.
* Tue Oct 24 2006 aj@suse.de
- Fix GNOME pattern.
* Tue Oct 24 2006 aj@suse.de
- Use opensync instead of multisync.
* Mon Oct 23 2006 aj@suse.de
- Rework GNOME and KDE patterns to reduce list of patterns
  significantly.
* Sun Oct 22 2006 aj@suse.de
- Add opensuse-update to kde-basis.
* Fri Oct 20 2006 aj@suse.de
- Make separate zmd pattern.
* Fri Oct 20 2006 aj@suse.de
- Remove / Merge patterns with just one package.
- i18n update.
* Mon Oct 16 2006 aj@suse.de
- Xfce is not on CD media.
* Fri Oct 13 2006 aj@suse.de
- Fix broken requires.
* Fri Oct 13 2006 aj@suse.de
- Fix requires on basis-devel (#212192).
* Thu Oct 12 2006 aj@suse.de
- Remove kpowersave from default install for now.
* Thu Oct 12 2006 aj@suse.de
- Add qtcurve packages.
* Thu Oct 12 2006 aj@suse.de
- Make imaging patterns hidden.
* Wed Oct 11 2006 aj@suse.de
- Add xorg-x11-libXext to X11 pattern.
* Wed Oct 11 2006 aj@suse.de
- Make Evolution recommended (#211175).
* Mon Oct  9 2006 aj@suse.de
- Add hplip to BASIS (#210984).
* Mon Oct  9 2006 aj@suse.de
- Handle some more i386/x86-64 only packages.
- Add hotkey-setup to LAPTOP (#210879).
- Translation update.
* Sun Oct  8 2006 aj@suse.de
- Add xdg-utils to X11 (#210884).
* Fri Oct  6 2006 aj@suse.de
- Add orca, gnopernicus (bug #208686).
- Adjust kde-games.
- localization update.
* Thu Oct  5 2006 aj@suse.de
- Fix GNOME-Multimedia pattern.
* Thu Oct  5 2006 aj@suse.de
- Add synergy to X11 pattern.
* Wed Oct  4 2006 aj@suse.de
- Add gnucash to GNOME-Office (#209719).
* Wed Oct  4 2006 aj@suse.de
- Remove devel-java from CDs.
* Tue Oct  3 2006 lrupp@suse.de
- fix typo in DEVEL-Java pattern
* Tue Oct  3 2006 aj@suse.de
- Update to 1.018 patterns.
- Improve i18n handling.
* Sun Oct  1 2006 aj@suse.de
- Rename BASIS-DEVEL to BASIS-DEVEL.
- Add DEVEL-RPM-Build-Environment Pattern (#209368).
* Sat Sep 30 2006 aj@suse.de
- Add various IDEs.
- Fix various bugreports about missing patterns (207916, 208384,
  208676).
- Merge KDE-HELP to KDE-Basis.
* Sat Sep 30 2006 aj@suse.de
- Add remote_desktop and devel_java patterns.
* Fri Sep 29 2006 aj@suse.de
- Add first support for extends/includes, use it in graphics
  pattern.
* Fri Sep 29 2006 aj@suse.de
- translation updates
- some packages added to patterns.
* Mon Sep 25 2006 aj@suse.de
- Enable subpackages.
* Sun Sep 24 2006 aj@suse.de
- Remove dropped packages.
- Rename packages.
* Sun Sep 24 2006 aj@suse.de
- Fix bugs in patterns.
- Disable subpackages for now.
* Sat Sep 23 2006 aj@suse.de
- Create subpackages for different media.
* Fri Sep 22 2006 aj@suse.de
- Add further development patterns:
  * Ruby
  * Ruby on Rails
  * YaST
  * Web
  * Python
  * Perl
- Rename SDK patterns to devel.
* Fri Sep 22 2006 aj@suse.de
- Update translations.
- Rework various patterns.
* Wed Sep 20 2006 aj@suse.de
- Translation updates.
* Wed Sep 20 2006 aj@suse.de
- Fix typo in kde pattern.
* Tue Sep 19 2006 aj@suse.de
- Adjust for package rename.
* Mon Sep 18 2006 aj@suse.de
- Update et, sv.
- Enhance GAMES, NETWORK-ADMIN patterns.
* Wed Sep 13 2006 aj@suse.de
- Add kdebase3-beagle to KDE-BASIS.
* Tue Sep 12 2006 aj@suse.de
- Fix Non-OSS pattern.
- Add cups to BASIS.
* Mon Sep 11 2006 aj@suse.de
- Add network-admin pattern.
* Mon Sep 11 2006 aj@suse.de
- Add non-OSS pattern.
* Fri Sep  8 2006 aj@suse.de
- Add further GNOME/KDE patterns.
* Fri Sep  8 2006 aj@suse.de
- Update translations, add further languages.
* Thu Sep  7 2006 aj@suse.de
- Further refine KDE and GNOME patterns.
- Refine BASE pattern.
* Wed Sep  6 2006 aj@suse.de
- Add GAMES.
* Wed Sep  6 2006 aj@suse.de
- Splitup new patterns from base.
* Wed Sep  6 2006 aj@suse.de
- Add gnome development.
* Mon Sep  4 2006 aj@suse.de
- Fix order of patterns.
- Make laptop patterns visible.
- Fix some bugs.
* Mon Sep  4 2006 aj@suse.de
- qt->libqt4.
* Fri Sep  1 2006 aj@suse.de
- Abort if errors occur in patterns.
- Split GNOME Patterns - thanks Gary Ekker!
- Add GNOME/KDE Laptop patterns.
* Wed Aug 30 2006 aj@suse.de
- Add xml pattern.
- Add xfce pattern.
- Add console pattern.
* Wed Aug 30 2006 aj@suse.de
- Add fonts pattern.
* Mon Aug 28 2006 aj@suse.de
- Further cleanup of files.
* Mon Aug 28 2006 aj@suse.de
- Fix translations for some languages.  Thanks to Jyri Palokangas
  for noticing.
- Do not emit untranslated strings.
* Mon Aug 28 2006 aj@suse.de
- Fix bash script.
* Mon Aug 28 2006 aj@suse.de
- Move pattern creation logic from spec file to new bash script.
- Fix gettext configure.
* Sat Aug 26 2006 aj@suse.de
- Fix spec file.
* Fri Aug 25 2006 aj@suse.de
- Add pam-config.
* Thu Aug 24 2006 aj@suse.de
- Add icons.
- Fix spec file.
* Thu Aug 24 2006 aj@suse.de
- Enhance patterns, fix imaging patterns.
* Thu Aug 24 2006 aj@suse.de
- Fix description of patterns - thanks Stephan Binner.
* Mon Aug 21 2006 aj@suse.de
- Add role "not yet classified" to clean up.
* Mon Aug 21 2006 aj@suse.de
- Add further patterns.
- Fix KDE patterns.
* Thu Aug 17 2006 aj@suse.de
- Enhance KDE-Office.
* Thu Aug 17 2006 aj@suse.de
- Enhance KDE patterns.
* Wed Aug 16 2006 aj@suse.de
- Add kdenetwork3 packages.
* Wed Aug 16 2006 aj@suse.de
- Add KDE subpatterns for testing.
* Tue Aug 15 2006 aj@suse.de
- Use pattern-openSUSE as packagename for po files.
* Thu Aug 10 2006 aj@suse.de
- Remove suggests apparmor from base to workaround Bug 198095.
* Wed Aug  9 2006 aj@suse.de
- Remove wrong icons for now.
* Tue Aug  8 2006 aj@suse.de
- Remove kdenetwork3-vnc from kde pattern.
- reorder imaging patterns
* Tue Aug  1 2006 aj@suse.de
- Changes for X.Org 7.1
- Add dhcp and dns server pattern.
- fix kde imaging pattern.
- Add first devel patterns.
* Mon Jul 31 2006 aj@suse.de
- Add imaging pattern.
* Sun Jul 30 2006 aj@suse.de
- Add beagle.
* Thu Jul 27 2006 aj@suse.de
- Cleanup theming.
* Wed Jul 26 2006 aj@suse.de
- Add and remove packages.
* Mon Jul 17 2006 aj@suse.de
- suse-release -> openSUSE-release.
* Mon Jul 17 2006 aj@suse.de
- Add NetworkManager packages to KDE/GNOME packages.
* Mon Jul 17 2006 aj@suse.de
- Adjust patterns.
- Change naming for openSUSE.
* Wed Jul 12 2006 aj@suse.de
- Initial Pattern for openSUSE.  This is just a really basic list
  to get us started and test that everything works.
